from enum import Enum
import math
import string


class MassFluxUnits(Enum):
        """
            MassFluxUnits enumeration
        """
        
        GramPerSecondPerSquareMeter = 'gram_per_second_per_square_meter'
        """
            
        """
        
        GramPerSecondPerSquareCentimeter = 'gram_per_second_per_square_centimeter'
        """
            
        """
        
        GramPerSecondPerSquareMillimeter = 'gram_per_second_per_square_millimeter'
        """
            
        """
        
        GramPerHourPerSquareMeter = 'gram_per_hour_per_square_meter'
        """
            
        """
        
        GramPerHourPerSquareCentimeter = 'gram_per_hour_per_square_centimeter'
        """
            
        """
        
        GramPerHourPerSquareMillimeter = 'gram_per_hour_per_square_millimeter'
        """
            
        """
        
        KilogramPerSecondPerSquareMeter = 'kilogram_per_second_per_square_meter'
        """
            
        """
        
        KilogramPerSecondPerSquareCentimeter = 'kilogram_per_second_per_square_centimeter'
        """
            
        """
        
        KilogramPerSecondPerSquareMillimeter = 'kilogram_per_second_per_square_millimeter'
        """
            
        """
        
        KilogramPerHourPerSquareMeter = 'kilogram_per_hour_per_square_meter'
        """
            
        """
        
        KilogramPerHourPerSquareCentimeter = 'kilogram_per_hour_per_square_centimeter'
        """
            
        """
        
        KilogramPerHourPerSquareMillimeter = 'kilogram_per_hour_per_square_millimeter'
        """
            
        """
        

class MassFlux:
    """
    Mass flux is the mass flow rate per unit area.

    Args:
        value (float): The value.
        from_unit (MassFluxUnits): The MassFlux unit to create from, The default unit is KilogramPerSecondPerSquareMeter
    """
    def __init__(self, value: float, from_unit: MassFluxUnits = MassFluxUnits.KilogramPerSecondPerSquareMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__grams_per_second_per_square_meter = None
        
        self.__grams_per_second_per_square_centimeter = None
        
        self.__grams_per_second_per_square_millimeter = None
        
        self.__grams_per_hour_per_square_meter = None
        
        self.__grams_per_hour_per_square_centimeter = None
        
        self.__grams_per_hour_per_square_millimeter = None
        
        self.__kilograms_per_second_per_square_meter = None
        
        self.__kilograms_per_second_per_square_centimeter = None
        
        self.__kilograms_per_second_per_square_millimeter = None
        
        self.__kilograms_per_hour_per_square_meter = None
        
        self.__kilograms_per_hour_per_square_centimeter = None
        
        self.__kilograms_per_hour_per_square_millimeter = None
        

    def __convert_from_base(self, from_unit: MassFluxUnits) -> float:
        value = self.__value
        
        if from_unit == MassFluxUnits.GramPerSecondPerSquareMeter:
            return (value * 1e3)
        
        if from_unit == MassFluxUnits.GramPerSecondPerSquareCentimeter:
            return (value * 1e-1)
        
        if from_unit == MassFluxUnits.GramPerSecondPerSquareMillimeter:
            return (value * 1e-3)
        
        if from_unit == MassFluxUnits.GramPerHourPerSquareMeter:
            return (value * 3.6e6)
        
        if from_unit == MassFluxUnits.GramPerHourPerSquareCentimeter:
            return (value * 3.6e2)
        
        if from_unit == MassFluxUnits.GramPerHourPerSquareMillimeter:
            return (value * 3.6e0)
        
        if from_unit == MassFluxUnits.KilogramPerSecondPerSquareMeter:
            return ((value * 1e3) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerSecondPerSquareCentimeter:
            return ((value * 1e-1) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerSecondPerSquareMillimeter:
            return ((value * 1e-3) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerHourPerSquareMeter:
            return ((value * 3.6e6) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerHourPerSquareCentimeter:
            return ((value * 3.6e2) / 1000.0)
        
        if from_unit == MassFluxUnits.KilogramPerHourPerSquareMillimeter:
            return ((value * 3.6e0) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MassFluxUnits) -> float:
        
        if to_unit == MassFluxUnits.GramPerSecondPerSquareMeter:
            return (value / 1e3)
        
        if to_unit == MassFluxUnits.GramPerSecondPerSquareCentimeter:
            return (value / 1e-1)
        
        if to_unit == MassFluxUnits.GramPerSecondPerSquareMillimeter:
            return (value / 1e-3)
        
        if to_unit == MassFluxUnits.GramPerHourPerSquareMeter:
            return (value / 3.6e6)
        
        if to_unit == MassFluxUnits.GramPerHourPerSquareCentimeter:
            return (value / 3.6e2)
        
        if to_unit == MassFluxUnits.GramPerHourPerSquareMillimeter:
            return (value / 3.6e0)
        
        if to_unit == MassFluxUnits.KilogramPerSecondPerSquareMeter:
            return ((value / 1e3) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerSecondPerSquareCentimeter:
            return ((value / 1e-1) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerSecondPerSquareMillimeter:
            return ((value / 1e-3) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerHourPerSquareMeter:
            return ((value / 3.6e6) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerHourPerSquareCentimeter:
            return ((value / 3.6e2) * 1000.0)
        
        if to_unit == MassFluxUnits.KilogramPerHourPerSquareMillimeter:
            return ((value / 3.6e0) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_grams_per_second_per_square_meter(grams_per_second_per_square_meter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_second_per_square_meter.

        

        :param meters: The MassFlux value in grams_per_second_per_square_meter.
        :type grams_per_second_per_square_meter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_second_per_square_meter, MassFluxUnits.GramPerSecondPerSquareMeter)

    
    @staticmethod
    def from_grams_per_second_per_square_centimeter(grams_per_second_per_square_centimeter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_second_per_square_centimeter.

        

        :param meters: The MassFlux value in grams_per_second_per_square_centimeter.
        :type grams_per_second_per_square_centimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_second_per_square_centimeter, MassFluxUnits.GramPerSecondPerSquareCentimeter)

    
    @staticmethod
    def from_grams_per_second_per_square_millimeter(grams_per_second_per_square_millimeter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_second_per_square_millimeter.

        

        :param meters: The MassFlux value in grams_per_second_per_square_millimeter.
        :type grams_per_second_per_square_millimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_second_per_square_millimeter, MassFluxUnits.GramPerSecondPerSquareMillimeter)

    
    @staticmethod
    def from_grams_per_hour_per_square_meter(grams_per_hour_per_square_meter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_hour_per_square_meter.

        

        :param meters: The MassFlux value in grams_per_hour_per_square_meter.
        :type grams_per_hour_per_square_meter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_hour_per_square_meter, MassFluxUnits.GramPerHourPerSquareMeter)

    
    @staticmethod
    def from_grams_per_hour_per_square_centimeter(grams_per_hour_per_square_centimeter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_hour_per_square_centimeter.

        

        :param meters: The MassFlux value in grams_per_hour_per_square_centimeter.
        :type grams_per_hour_per_square_centimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_hour_per_square_centimeter, MassFluxUnits.GramPerHourPerSquareCentimeter)

    
    @staticmethod
    def from_grams_per_hour_per_square_millimeter(grams_per_hour_per_square_millimeter: float):
        """
        Create a new instance of MassFlux from a value in grams_per_hour_per_square_millimeter.

        

        :param meters: The MassFlux value in grams_per_hour_per_square_millimeter.
        :type grams_per_hour_per_square_millimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(grams_per_hour_per_square_millimeter, MassFluxUnits.GramPerHourPerSquareMillimeter)

    
    @staticmethod
    def from_kilograms_per_second_per_square_meter(kilograms_per_second_per_square_meter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_second_per_square_meter.

        

        :param meters: The MassFlux value in kilograms_per_second_per_square_meter.
        :type kilograms_per_second_per_square_meter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_second_per_square_meter, MassFluxUnits.KilogramPerSecondPerSquareMeter)

    
    @staticmethod
    def from_kilograms_per_second_per_square_centimeter(kilograms_per_second_per_square_centimeter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_second_per_square_centimeter.

        

        :param meters: The MassFlux value in kilograms_per_second_per_square_centimeter.
        :type kilograms_per_second_per_square_centimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_second_per_square_centimeter, MassFluxUnits.KilogramPerSecondPerSquareCentimeter)

    
    @staticmethod
    def from_kilograms_per_second_per_square_millimeter(kilograms_per_second_per_square_millimeter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_second_per_square_millimeter.

        

        :param meters: The MassFlux value in kilograms_per_second_per_square_millimeter.
        :type kilograms_per_second_per_square_millimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_second_per_square_millimeter, MassFluxUnits.KilogramPerSecondPerSquareMillimeter)

    
    @staticmethod
    def from_kilograms_per_hour_per_square_meter(kilograms_per_hour_per_square_meter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_hour_per_square_meter.

        

        :param meters: The MassFlux value in kilograms_per_hour_per_square_meter.
        :type kilograms_per_hour_per_square_meter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_hour_per_square_meter, MassFluxUnits.KilogramPerHourPerSquareMeter)

    
    @staticmethod
    def from_kilograms_per_hour_per_square_centimeter(kilograms_per_hour_per_square_centimeter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_hour_per_square_centimeter.

        

        :param meters: The MassFlux value in kilograms_per_hour_per_square_centimeter.
        :type kilograms_per_hour_per_square_centimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_hour_per_square_centimeter, MassFluxUnits.KilogramPerHourPerSquareCentimeter)

    
    @staticmethod
    def from_kilograms_per_hour_per_square_millimeter(kilograms_per_hour_per_square_millimeter: float):
        """
        Create a new instance of MassFlux from a value in kilograms_per_hour_per_square_millimeter.

        

        :param meters: The MassFlux value in kilograms_per_hour_per_square_millimeter.
        :type kilograms_per_hour_per_square_millimeter: float
        :return: A new instance of MassFlux.
        :rtype: MassFlux
        """
        return MassFlux(kilograms_per_hour_per_square_millimeter, MassFluxUnits.KilogramPerHourPerSquareMillimeter)

    
    @property
    def grams_per_second_per_square_meter(self) -> float:
        """
        
        """
        if self.__grams_per_second_per_square_meter != None:
            return self.__grams_per_second_per_square_meter
        self.__grams_per_second_per_square_meter = self.__convert_from_base(MassFluxUnits.GramPerSecondPerSquareMeter)
        return self.__grams_per_second_per_square_meter

    
    @property
    def grams_per_second_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__grams_per_second_per_square_centimeter != None:
            return self.__grams_per_second_per_square_centimeter
        self.__grams_per_second_per_square_centimeter = self.__convert_from_base(MassFluxUnits.GramPerSecondPerSquareCentimeter)
        return self.__grams_per_second_per_square_centimeter

    
    @property
    def grams_per_second_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__grams_per_second_per_square_millimeter != None:
            return self.__grams_per_second_per_square_millimeter
        self.__grams_per_second_per_square_millimeter = self.__convert_from_base(MassFluxUnits.GramPerSecondPerSquareMillimeter)
        return self.__grams_per_second_per_square_millimeter

    
    @property
    def grams_per_hour_per_square_meter(self) -> float:
        """
        
        """
        if self.__grams_per_hour_per_square_meter != None:
            return self.__grams_per_hour_per_square_meter
        self.__grams_per_hour_per_square_meter = self.__convert_from_base(MassFluxUnits.GramPerHourPerSquareMeter)
        return self.__grams_per_hour_per_square_meter

    
    @property
    def grams_per_hour_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__grams_per_hour_per_square_centimeter != None:
            return self.__grams_per_hour_per_square_centimeter
        self.__grams_per_hour_per_square_centimeter = self.__convert_from_base(MassFluxUnits.GramPerHourPerSquareCentimeter)
        return self.__grams_per_hour_per_square_centimeter

    
    @property
    def grams_per_hour_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__grams_per_hour_per_square_millimeter != None:
            return self.__grams_per_hour_per_square_millimeter
        self.__grams_per_hour_per_square_millimeter = self.__convert_from_base(MassFluxUnits.GramPerHourPerSquareMillimeter)
        return self.__grams_per_hour_per_square_millimeter

    
    @property
    def kilograms_per_second_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilograms_per_second_per_square_meter != None:
            return self.__kilograms_per_second_per_square_meter
        self.__kilograms_per_second_per_square_meter = self.__convert_from_base(MassFluxUnits.KilogramPerSecondPerSquareMeter)
        return self.__kilograms_per_second_per_square_meter

    
    @property
    def kilograms_per_second_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__kilograms_per_second_per_square_centimeter != None:
            return self.__kilograms_per_second_per_square_centimeter
        self.__kilograms_per_second_per_square_centimeter = self.__convert_from_base(MassFluxUnits.KilogramPerSecondPerSquareCentimeter)
        return self.__kilograms_per_second_per_square_centimeter

    
    @property
    def kilograms_per_second_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__kilograms_per_second_per_square_millimeter != None:
            return self.__kilograms_per_second_per_square_millimeter
        self.__kilograms_per_second_per_square_millimeter = self.__convert_from_base(MassFluxUnits.KilogramPerSecondPerSquareMillimeter)
        return self.__kilograms_per_second_per_square_millimeter

    
    @property
    def kilograms_per_hour_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilograms_per_hour_per_square_meter != None:
            return self.__kilograms_per_hour_per_square_meter
        self.__kilograms_per_hour_per_square_meter = self.__convert_from_base(MassFluxUnits.KilogramPerHourPerSquareMeter)
        return self.__kilograms_per_hour_per_square_meter

    
    @property
    def kilograms_per_hour_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__kilograms_per_hour_per_square_centimeter != None:
            return self.__kilograms_per_hour_per_square_centimeter
        self.__kilograms_per_hour_per_square_centimeter = self.__convert_from_base(MassFluxUnits.KilogramPerHourPerSquareCentimeter)
        return self.__kilograms_per_hour_per_square_centimeter

    
    @property
    def kilograms_per_hour_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__kilograms_per_hour_per_square_millimeter != None:
            return self.__kilograms_per_hour_per_square_millimeter
        self.__kilograms_per_hour_per_square_millimeter = self.__convert_from_base(MassFluxUnits.KilogramPerHourPerSquareMillimeter)
        return self.__kilograms_per_hour_per_square_millimeter

    
    def to_string(self, unit: MassFluxUnits = MassFluxUnits.KilogramPerSecondPerSquareMeter) -> string:
        """
        Format the MassFlux to string.
        Note! the default format for MassFlux is KilogramPerSecondPerSquareMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == MassFluxUnits.GramPerSecondPerSquareMeter:
            return f"""{self.grams_per_second_per_square_meter} g·s⁻¹·m⁻²"""
        
        if unit == MassFluxUnits.GramPerSecondPerSquareCentimeter:
            return f"""{self.grams_per_second_per_square_centimeter} g·s⁻¹·cm⁻²"""
        
        if unit == MassFluxUnits.GramPerSecondPerSquareMillimeter:
            return f"""{self.grams_per_second_per_square_millimeter} g·s⁻¹·mm⁻²"""
        
        if unit == MassFluxUnits.GramPerHourPerSquareMeter:
            return f"""{self.grams_per_hour_per_square_meter} g·h⁻¹·m⁻²"""
        
        if unit == MassFluxUnits.GramPerHourPerSquareCentimeter:
            return f"""{self.grams_per_hour_per_square_centimeter} g·h⁻¹·cm⁻²"""
        
        if unit == MassFluxUnits.GramPerHourPerSquareMillimeter:
            return f"""{self.grams_per_hour_per_square_millimeter} g·h⁻¹·mm⁻²"""
        
        if unit == MassFluxUnits.KilogramPerSecondPerSquareMeter:
            return f"""{self.kilograms_per_second_per_square_meter} """
        
        if unit == MassFluxUnits.KilogramPerSecondPerSquareCentimeter:
            return f"""{self.kilograms_per_second_per_square_centimeter} """
        
        if unit == MassFluxUnits.KilogramPerSecondPerSquareMillimeter:
            return f"""{self.kilograms_per_second_per_square_millimeter} """
        
        if unit == MassFluxUnits.KilogramPerHourPerSquareMeter:
            return f"""{self.kilograms_per_hour_per_square_meter} """
        
        if unit == MassFluxUnits.KilogramPerHourPerSquareCentimeter:
            return f"""{self.kilograms_per_hour_per_square_centimeter} """
        
        if unit == MassFluxUnits.KilogramPerHourPerSquareMillimeter:
            return f"""{self.kilograms_per_hour_per_square_millimeter} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: MassFluxUnits = MassFluxUnits.KilogramPerSecondPerSquareMeter) -> string:
        """
        Get MassFlux unit abbreviation.
        Note! the default abbreviation for MassFlux is KilogramPerSecondPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MassFluxUnits.GramPerSecondPerSquareMeter:
            return """g·s⁻¹·m⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerSecondPerSquareCentimeter:
            return """g·s⁻¹·cm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerSecondPerSquareMillimeter:
            return """g·s⁻¹·mm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerHourPerSquareMeter:
            return """g·h⁻¹·m⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerHourPerSquareCentimeter:
            return """g·h⁻¹·cm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.GramPerHourPerSquareMillimeter:
            return """g·h⁻¹·mm⁻²"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerSecondPerSquareMeter:
            return """"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerSecondPerSquareCentimeter:
            return """"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerSecondPerSquareMillimeter:
            return """"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerHourPerSquareMeter:
            return """"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerHourPerSquareCentimeter:
            return """"""
        
        if unit_abbreviation == MassFluxUnits.KilogramPerHourPerSquareMillimeter:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for +: 'MassFlux' and '{}'".format(type(other).__name__))
        return MassFlux(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for *: 'MassFlux' and '{}'".format(type(other).__name__))
        return MassFlux(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for -: 'MassFlux' and '{}'".format(type(other).__name__))
        return MassFlux(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for /: 'MassFlux' and '{}'".format(type(other).__name__))
        return MassFlux(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for %: 'MassFlux' and '{}'".format(type(other).__name__))
        return MassFlux(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for **: 'MassFlux' and '{}'".format(type(other).__name__))
        return MassFlux(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for ==: 'MassFlux' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for <: 'MassFlux' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for >: 'MassFlux' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for <=: 'MassFlux' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, MassFlux):
            raise TypeError("unsupported operand type(s) for >=: 'MassFlux' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class AreaMomentOfInertiaUnits(Enum):
        """
            AreaMomentOfInertiaUnits enumeration
        """
        
        MeterToTheFourth = 'meter_to_the_fourth'
        """
            
        """
        
        DecimeterToTheFourth = 'decimeter_to_the_fourth'
        """
            
        """
        
        CentimeterToTheFourth = 'centimeter_to_the_fourth'
        """
            
        """
        
        MillimeterToTheFourth = 'millimeter_to_the_fourth'
        """
            
        """
        
        FootToTheFourth = 'foot_to_the_fourth'
        """
            
        """
        
        InchToTheFourth = 'inch_to_the_fourth'
        """
            
        """
        

class AreaMomentOfInertia:
    """
    A geometric property of an area that reflects how its points are distributed with regard to an axis.

    Args:
        value (float): The value.
        from_unit (AreaMomentOfInertiaUnits): The AreaMomentOfInertia unit to create from, The default unit is MeterToTheFourth
    """
    def __init__(self, value: float, from_unit: AreaMomentOfInertiaUnits = AreaMomentOfInertiaUnits.MeterToTheFourth):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__meters_to_the_fourth = None
        
        self.__decimeters_to_the_fourth = None
        
        self.__centimeters_to_the_fourth = None
        
        self.__millimeters_to_the_fourth = None
        
        self.__feet_to_the_fourth = None
        
        self.__inches_to_the_fourth = None
        

    def __convert_from_base(self, from_unit: AreaMomentOfInertiaUnits) -> float:
        value = self.__value
        
        if from_unit == AreaMomentOfInertiaUnits.MeterToTheFourth:
            return (value)
        
        if from_unit == AreaMomentOfInertiaUnits.DecimeterToTheFourth:
            return (value * 1e4)
        
        if from_unit == AreaMomentOfInertiaUnits.CentimeterToTheFourth:
            return (value * 1e8)
        
        if from_unit == AreaMomentOfInertiaUnits.MillimeterToTheFourth:
            return (value * 1e12)
        
        if from_unit == AreaMomentOfInertiaUnits.FootToTheFourth:
            return (value / math.pow(0.3048, 4))
        
        if from_unit == AreaMomentOfInertiaUnits.InchToTheFourth:
            return (value / math.pow(2.54e-2, 4))
        
        return None


    def __convert_to_base(self, value: float, to_unit: AreaMomentOfInertiaUnits) -> float:
        
        if to_unit == AreaMomentOfInertiaUnits.MeterToTheFourth:
            return (value)
        
        if to_unit == AreaMomentOfInertiaUnits.DecimeterToTheFourth:
            return (value / 1e4)
        
        if to_unit == AreaMomentOfInertiaUnits.CentimeterToTheFourth:
            return (value / 1e8)
        
        if to_unit == AreaMomentOfInertiaUnits.MillimeterToTheFourth:
            return (value / 1e12)
        
        if to_unit == AreaMomentOfInertiaUnits.FootToTheFourth:
            return (value * math.pow(0.3048, 4))
        
        if to_unit == AreaMomentOfInertiaUnits.InchToTheFourth:
            return (value * math.pow(2.54e-2, 4))
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_meters_to_the_fourth(meters_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in meters_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in meters_to_the_fourth.
        :type meters_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(meters_to_the_fourth, AreaMomentOfInertiaUnits.MeterToTheFourth)

    
    @staticmethod
    def from_decimeters_to_the_fourth(decimeters_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in decimeters_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in decimeters_to_the_fourth.
        :type decimeters_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(decimeters_to_the_fourth, AreaMomentOfInertiaUnits.DecimeterToTheFourth)

    
    @staticmethod
    def from_centimeters_to_the_fourth(centimeters_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in centimeters_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in centimeters_to_the_fourth.
        :type centimeters_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(centimeters_to_the_fourth, AreaMomentOfInertiaUnits.CentimeterToTheFourth)

    
    @staticmethod
    def from_millimeters_to_the_fourth(millimeters_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in millimeters_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in millimeters_to_the_fourth.
        :type millimeters_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(millimeters_to_the_fourth, AreaMomentOfInertiaUnits.MillimeterToTheFourth)

    
    @staticmethod
    def from_feet_to_the_fourth(feet_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in feet_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in feet_to_the_fourth.
        :type feet_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(feet_to_the_fourth, AreaMomentOfInertiaUnits.FootToTheFourth)

    
    @staticmethod
    def from_inches_to_the_fourth(inches_to_the_fourth: float):
        """
        Create a new instance of AreaMomentOfInertia from a value in inches_to_the_fourth.

        

        :param meters: The AreaMomentOfInertia value in inches_to_the_fourth.
        :type inches_to_the_fourth: float
        :return: A new instance of AreaMomentOfInertia.
        :rtype: AreaMomentOfInertia
        """
        return AreaMomentOfInertia(inches_to_the_fourth, AreaMomentOfInertiaUnits.InchToTheFourth)

    
    @property
    def meters_to_the_fourth(self) -> float:
        """
        
        """
        if self.__meters_to_the_fourth != None:
            return self.__meters_to_the_fourth
        self.__meters_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.MeterToTheFourth)
        return self.__meters_to_the_fourth

    
    @property
    def decimeters_to_the_fourth(self) -> float:
        """
        
        """
        if self.__decimeters_to_the_fourth != None:
            return self.__decimeters_to_the_fourth
        self.__decimeters_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.DecimeterToTheFourth)
        return self.__decimeters_to_the_fourth

    
    @property
    def centimeters_to_the_fourth(self) -> float:
        """
        
        """
        if self.__centimeters_to_the_fourth != None:
            return self.__centimeters_to_the_fourth
        self.__centimeters_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.CentimeterToTheFourth)
        return self.__centimeters_to_the_fourth

    
    @property
    def millimeters_to_the_fourth(self) -> float:
        """
        
        """
        if self.__millimeters_to_the_fourth != None:
            return self.__millimeters_to_the_fourth
        self.__millimeters_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.MillimeterToTheFourth)
        return self.__millimeters_to_the_fourth

    
    @property
    def feet_to_the_fourth(self) -> float:
        """
        
        """
        if self.__feet_to_the_fourth != None:
            return self.__feet_to_the_fourth
        self.__feet_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.FootToTheFourth)
        return self.__feet_to_the_fourth

    
    @property
    def inches_to_the_fourth(self) -> float:
        """
        
        """
        if self.__inches_to_the_fourth != None:
            return self.__inches_to_the_fourth
        self.__inches_to_the_fourth = self.__convert_from_base(AreaMomentOfInertiaUnits.InchToTheFourth)
        return self.__inches_to_the_fourth

    
    def to_string(self, unit: AreaMomentOfInertiaUnits = AreaMomentOfInertiaUnits.MeterToTheFourth) -> string:
        """
        Format the AreaMomentOfInertia to string.
        Note! the default format for AreaMomentOfInertia is MeterToTheFourth.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == AreaMomentOfInertiaUnits.MeterToTheFourth:
            return f"""{self.meters_to_the_fourth} m⁴"""
        
        if unit == AreaMomentOfInertiaUnits.DecimeterToTheFourth:
            return f"""{self.decimeters_to_the_fourth} dm⁴"""
        
        if unit == AreaMomentOfInertiaUnits.CentimeterToTheFourth:
            return f"""{self.centimeters_to_the_fourth} cm⁴"""
        
        if unit == AreaMomentOfInertiaUnits.MillimeterToTheFourth:
            return f"""{self.millimeters_to_the_fourth} mm⁴"""
        
        if unit == AreaMomentOfInertiaUnits.FootToTheFourth:
            return f"""{self.feet_to_the_fourth} ft⁴"""
        
        if unit == AreaMomentOfInertiaUnits.InchToTheFourth:
            return f"""{self.inches_to_the_fourth} in⁴"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: AreaMomentOfInertiaUnits = AreaMomentOfInertiaUnits.MeterToTheFourth) -> string:
        """
        Get AreaMomentOfInertia unit abbreviation.
        Note! the default abbreviation for AreaMomentOfInertia is MeterToTheFourth.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.MeterToTheFourth:
            return """m⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.DecimeterToTheFourth:
            return """dm⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.CentimeterToTheFourth:
            return """cm⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.MillimeterToTheFourth:
            return """mm⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.FootToTheFourth:
            return """ft⁴"""
        
        if unit_abbreviation == AreaMomentOfInertiaUnits.InchToTheFourth:
            return """in⁴"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for +: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return AreaMomentOfInertia(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for *: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return AreaMomentOfInertia(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for -: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return AreaMomentOfInertia(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for /: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return AreaMomentOfInertia(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for %: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return AreaMomentOfInertia(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for **: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return AreaMomentOfInertia(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for ==: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for <: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for >: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for <=: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, AreaMomentOfInertia):
            raise TypeError("unsupported operand type(s) for >=: 'AreaMomentOfInertia' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class TorqueUnits(Enum):
        """
            TorqueUnits enumeration
        """
        
        NewtonMillimeter = 'newton_millimeter'
        """
            
        """
        
        NewtonCentimeter = 'newton_centimeter'
        """
            
        """
        
        NewtonMeter = 'newton_meter'
        """
            
        """
        
        PoundalFoot = 'poundal_foot'
        """
            
        """
        
        PoundForceInch = 'pound_force_inch'
        """
            
        """
        
        PoundForceFoot = 'pound_force_foot'
        """
            
        """
        
        GramForceMillimeter = 'gram_force_millimeter'
        """
            
        """
        
        GramForceCentimeter = 'gram_force_centimeter'
        """
            
        """
        
        GramForceMeter = 'gram_force_meter'
        """
            
        """
        
        KilogramForceMillimeter = 'kilogram_force_millimeter'
        """
            
        """
        
        KilogramForceCentimeter = 'kilogram_force_centimeter'
        """
            
        """
        
        KilogramForceMeter = 'kilogram_force_meter'
        """
            
        """
        
        TonneForceMillimeter = 'tonne_force_millimeter'
        """
            
        """
        
        TonneForceCentimeter = 'tonne_force_centimeter'
        """
            
        """
        
        TonneForceMeter = 'tonne_force_meter'
        """
            
        """
        
        KilonewtonMillimeter = 'kilonewton_millimeter'
        """
            
        """
        
        MeganewtonMillimeter = 'meganewton_millimeter'
        """
            
        """
        
        KilonewtonCentimeter = 'kilonewton_centimeter'
        """
            
        """
        
        MeganewtonCentimeter = 'meganewton_centimeter'
        """
            
        """
        
        KilonewtonMeter = 'kilonewton_meter'
        """
            
        """
        
        MeganewtonMeter = 'meganewton_meter'
        """
            
        """
        
        KilopoundForceInch = 'kilopound_force_inch'
        """
            
        """
        
        MegapoundForceInch = 'megapound_force_inch'
        """
            
        """
        
        KilopoundForceFoot = 'kilopound_force_foot'
        """
            
        """
        
        MegapoundForceFoot = 'megapound_force_foot'
        """
            
        """
        

class Torque:
    """
    Torque, moment or moment of force (see the terminology below), is the tendency of a force to rotate an object about an axis,[1] fulcrum, or pivot. Just as a force is a push or a pull, a torque can be thought of as a twist to an object. Mathematically, torque is defined as the cross product of the lever-arm distance and force, which tends to produce rotation. Loosely speaking, torque is a measure of the turning force on an object such as a bolt or a flywheel. For example, pushing or pulling the handle of a wrench connected to a nut or bolt produces a torque (turning force) that loosens or tightens the nut or bolt.

    Args:
        value (float): The value.
        from_unit (TorqueUnits): The Torque unit to create from, The default unit is NewtonMeter
    """
    def __init__(self, value: float, from_unit: TorqueUnits = TorqueUnits.NewtonMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__newton_millimeters = None
        
        self.__newton_centimeters = None
        
        self.__newton_meters = None
        
        self.__poundal_feet = None
        
        self.__pound_force_inches = None
        
        self.__pound_force_feet = None
        
        self.__gram_force_millimeters = None
        
        self.__gram_force_centimeters = None
        
        self.__gram_force_meters = None
        
        self.__kilogram_force_millimeters = None
        
        self.__kilogram_force_centimeters = None
        
        self.__kilogram_force_meters = None
        
        self.__tonne_force_millimeters = None
        
        self.__tonne_force_centimeters = None
        
        self.__tonne_force_meters = None
        
        self.__kilonewton_millimeters = None
        
        self.__meganewton_millimeters = None
        
        self.__kilonewton_centimeters = None
        
        self.__meganewton_centimeters = None
        
        self.__kilonewton_meters = None
        
        self.__meganewton_meters = None
        
        self.__kilopound_force_inches = None
        
        self.__megapound_force_inches = None
        
        self.__kilopound_force_feet = None
        
        self.__megapound_force_feet = None
        

    def __convert_from_base(self, from_unit: TorqueUnits) -> float:
        value = self.__value
        
        if from_unit == TorqueUnits.NewtonMillimeter:
            return (value * 1000)
        
        if from_unit == TorqueUnits.NewtonCentimeter:
            return (value * 100)
        
        if from_unit == TorqueUnits.NewtonMeter:
            return (value)
        
        if from_unit == TorqueUnits.PoundalFoot:
            return (value / 4.21401100938048e-2)
        
        if from_unit == TorqueUnits.PoundForceInch:
            return (value / 1.129848290276167e-1)
        
        if from_unit == TorqueUnits.PoundForceFoot:
            return (value / 1.3558179483314)
        
        if from_unit == TorqueUnits.GramForceMillimeter:
            return (value / 9.80665e-6)
        
        if from_unit == TorqueUnits.GramForceCentimeter:
            return (value / 9.80665e-5)
        
        if from_unit == TorqueUnits.GramForceMeter:
            return (value / 9.80665e-3)
        
        if from_unit == TorqueUnits.KilogramForceMillimeter:
            return (value / 9.80665e-3)
        
        if from_unit == TorqueUnits.KilogramForceCentimeter:
            return (value / 9.80665e-2)
        
        if from_unit == TorqueUnits.KilogramForceMeter:
            return (value / 9.80665)
        
        if from_unit == TorqueUnits.TonneForceMillimeter:
            return (value / 9.80665)
        
        if from_unit == TorqueUnits.TonneForceCentimeter:
            return (value / 9.80665e1)
        
        if from_unit == TorqueUnits.TonneForceMeter:
            return (value / 9.80665e3)
        
        if from_unit == TorqueUnits.KilonewtonMillimeter:
            return ((value * 1000) / 1000.0)
        
        if from_unit == TorqueUnits.MeganewtonMillimeter:
            return ((value * 1000) / 1000000.0)
        
        if from_unit == TorqueUnits.KilonewtonCentimeter:
            return ((value * 100) / 1000.0)
        
        if from_unit == TorqueUnits.MeganewtonCentimeter:
            return ((value * 100) / 1000000.0)
        
        if from_unit == TorqueUnits.KilonewtonMeter:
            return ((value) / 1000.0)
        
        if from_unit == TorqueUnits.MeganewtonMeter:
            return ((value) / 1000000.0)
        
        if from_unit == TorqueUnits.KilopoundForceInch:
            return ((value / 1.129848290276167e-1) / 1000.0)
        
        if from_unit == TorqueUnits.MegapoundForceInch:
            return ((value / 1.129848290276167e-1) / 1000000.0)
        
        if from_unit == TorqueUnits.KilopoundForceFoot:
            return ((value / 1.3558179483314) / 1000.0)
        
        if from_unit == TorqueUnits.MegapoundForceFoot:
            return ((value / 1.3558179483314) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TorqueUnits) -> float:
        
        if to_unit == TorqueUnits.NewtonMillimeter:
            return (value * 0.001)
        
        if to_unit == TorqueUnits.NewtonCentimeter:
            return (value * 0.01)
        
        if to_unit == TorqueUnits.NewtonMeter:
            return (value)
        
        if to_unit == TorqueUnits.PoundalFoot:
            return (value * 4.21401100938048e-2)
        
        if to_unit == TorqueUnits.PoundForceInch:
            return (value * 1.129848290276167e-1)
        
        if to_unit == TorqueUnits.PoundForceFoot:
            return (value * 1.3558179483314)
        
        if to_unit == TorqueUnits.GramForceMillimeter:
            return (value * 9.80665e-6)
        
        if to_unit == TorqueUnits.GramForceCentimeter:
            return (value * 9.80665e-5)
        
        if to_unit == TorqueUnits.GramForceMeter:
            return (value * 9.80665e-3)
        
        if to_unit == TorqueUnits.KilogramForceMillimeter:
            return (value * 9.80665e-3)
        
        if to_unit == TorqueUnits.KilogramForceCentimeter:
            return (value * 9.80665e-2)
        
        if to_unit == TorqueUnits.KilogramForceMeter:
            return (value * 9.80665)
        
        if to_unit == TorqueUnits.TonneForceMillimeter:
            return (value * 9.80665)
        
        if to_unit == TorqueUnits.TonneForceCentimeter:
            return (value * 9.80665e1)
        
        if to_unit == TorqueUnits.TonneForceMeter:
            return (value * 9.80665e3)
        
        if to_unit == TorqueUnits.KilonewtonMillimeter:
            return ((value * 0.001) * 1000.0)
        
        if to_unit == TorqueUnits.MeganewtonMillimeter:
            return ((value * 0.001) * 1000000.0)
        
        if to_unit == TorqueUnits.KilonewtonCentimeter:
            return ((value * 0.01) * 1000.0)
        
        if to_unit == TorqueUnits.MeganewtonCentimeter:
            return ((value * 0.01) * 1000000.0)
        
        if to_unit == TorqueUnits.KilonewtonMeter:
            return ((value) * 1000.0)
        
        if to_unit == TorqueUnits.MeganewtonMeter:
            return ((value) * 1000000.0)
        
        if to_unit == TorqueUnits.KilopoundForceInch:
            return ((value * 1.129848290276167e-1) * 1000.0)
        
        if to_unit == TorqueUnits.MegapoundForceInch:
            return ((value * 1.129848290276167e-1) * 1000000.0)
        
        if to_unit == TorqueUnits.KilopoundForceFoot:
            return ((value * 1.3558179483314) * 1000.0)
        
        if to_unit == TorqueUnits.MegapoundForceFoot:
            return ((value * 1.3558179483314) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_newton_millimeters(newton_millimeters: float):
        """
        Create a new instance of Torque from a value in newton_millimeters.

        

        :param meters: The Torque value in newton_millimeters.
        :type newton_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(newton_millimeters, TorqueUnits.NewtonMillimeter)

    
    @staticmethod
    def from_newton_centimeters(newton_centimeters: float):
        """
        Create a new instance of Torque from a value in newton_centimeters.

        

        :param meters: The Torque value in newton_centimeters.
        :type newton_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(newton_centimeters, TorqueUnits.NewtonCentimeter)

    
    @staticmethod
    def from_newton_meters(newton_meters: float):
        """
        Create a new instance of Torque from a value in newton_meters.

        

        :param meters: The Torque value in newton_meters.
        :type newton_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(newton_meters, TorqueUnits.NewtonMeter)

    
    @staticmethod
    def from_poundal_feet(poundal_feet: float):
        """
        Create a new instance of Torque from a value in poundal_feet.

        

        :param meters: The Torque value in poundal_feet.
        :type poundal_feet: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(poundal_feet, TorqueUnits.PoundalFoot)

    
    @staticmethod
    def from_pound_force_inches(pound_force_inches: float):
        """
        Create a new instance of Torque from a value in pound_force_inches.

        

        :param meters: The Torque value in pound_force_inches.
        :type pound_force_inches: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(pound_force_inches, TorqueUnits.PoundForceInch)

    
    @staticmethod
    def from_pound_force_feet(pound_force_feet: float):
        """
        Create a new instance of Torque from a value in pound_force_feet.

        

        :param meters: The Torque value in pound_force_feet.
        :type pound_force_feet: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(pound_force_feet, TorqueUnits.PoundForceFoot)

    
    @staticmethod
    def from_gram_force_millimeters(gram_force_millimeters: float):
        """
        Create a new instance of Torque from a value in gram_force_millimeters.

        

        :param meters: The Torque value in gram_force_millimeters.
        :type gram_force_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(gram_force_millimeters, TorqueUnits.GramForceMillimeter)

    
    @staticmethod
    def from_gram_force_centimeters(gram_force_centimeters: float):
        """
        Create a new instance of Torque from a value in gram_force_centimeters.

        

        :param meters: The Torque value in gram_force_centimeters.
        :type gram_force_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(gram_force_centimeters, TorqueUnits.GramForceCentimeter)

    
    @staticmethod
    def from_gram_force_meters(gram_force_meters: float):
        """
        Create a new instance of Torque from a value in gram_force_meters.

        

        :param meters: The Torque value in gram_force_meters.
        :type gram_force_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(gram_force_meters, TorqueUnits.GramForceMeter)

    
    @staticmethod
    def from_kilogram_force_millimeters(kilogram_force_millimeters: float):
        """
        Create a new instance of Torque from a value in kilogram_force_millimeters.

        

        :param meters: The Torque value in kilogram_force_millimeters.
        :type kilogram_force_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilogram_force_millimeters, TorqueUnits.KilogramForceMillimeter)

    
    @staticmethod
    def from_kilogram_force_centimeters(kilogram_force_centimeters: float):
        """
        Create a new instance of Torque from a value in kilogram_force_centimeters.

        

        :param meters: The Torque value in kilogram_force_centimeters.
        :type kilogram_force_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilogram_force_centimeters, TorqueUnits.KilogramForceCentimeter)

    
    @staticmethod
    def from_kilogram_force_meters(kilogram_force_meters: float):
        """
        Create a new instance of Torque from a value in kilogram_force_meters.

        

        :param meters: The Torque value in kilogram_force_meters.
        :type kilogram_force_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilogram_force_meters, TorqueUnits.KilogramForceMeter)

    
    @staticmethod
    def from_tonne_force_millimeters(tonne_force_millimeters: float):
        """
        Create a new instance of Torque from a value in tonne_force_millimeters.

        

        :param meters: The Torque value in tonne_force_millimeters.
        :type tonne_force_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(tonne_force_millimeters, TorqueUnits.TonneForceMillimeter)

    
    @staticmethod
    def from_tonne_force_centimeters(tonne_force_centimeters: float):
        """
        Create a new instance of Torque from a value in tonne_force_centimeters.

        

        :param meters: The Torque value in tonne_force_centimeters.
        :type tonne_force_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(tonne_force_centimeters, TorqueUnits.TonneForceCentimeter)

    
    @staticmethod
    def from_tonne_force_meters(tonne_force_meters: float):
        """
        Create a new instance of Torque from a value in tonne_force_meters.

        

        :param meters: The Torque value in tonne_force_meters.
        :type tonne_force_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(tonne_force_meters, TorqueUnits.TonneForceMeter)

    
    @staticmethod
    def from_kilonewton_millimeters(kilonewton_millimeters: float):
        """
        Create a new instance of Torque from a value in kilonewton_millimeters.

        

        :param meters: The Torque value in kilonewton_millimeters.
        :type kilonewton_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilonewton_millimeters, TorqueUnits.KilonewtonMillimeter)

    
    @staticmethod
    def from_meganewton_millimeters(meganewton_millimeters: float):
        """
        Create a new instance of Torque from a value in meganewton_millimeters.

        

        :param meters: The Torque value in meganewton_millimeters.
        :type meganewton_millimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(meganewton_millimeters, TorqueUnits.MeganewtonMillimeter)

    
    @staticmethod
    def from_kilonewton_centimeters(kilonewton_centimeters: float):
        """
        Create a new instance of Torque from a value in kilonewton_centimeters.

        

        :param meters: The Torque value in kilonewton_centimeters.
        :type kilonewton_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilonewton_centimeters, TorqueUnits.KilonewtonCentimeter)

    
    @staticmethod
    def from_meganewton_centimeters(meganewton_centimeters: float):
        """
        Create a new instance of Torque from a value in meganewton_centimeters.

        

        :param meters: The Torque value in meganewton_centimeters.
        :type meganewton_centimeters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(meganewton_centimeters, TorqueUnits.MeganewtonCentimeter)

    
    @staticmethod
    def from_kilonewton_meters(kilonewton_meters: float):
        """
        Create a new instance of Torque from a value in kilonewton_meters.

        

        :param meters: The Torque value in kilonewton_meters.
        :type kilonewton_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilonewton_meters, TorqueUnits.KilonewtonMeter)

    
    @staticmethod
    def from_meganewton_meters(meganewton_meters: float):
        """
        Create a new instance of Torque from a value in meganewton_meters.

        

        :param meters: The Torque value in meganewton_meters.
        :type meganewton_meters: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(meganewton_meters, TorqueUnits.MeganewtonMeter)

    
    @staticmethod
    def from_kilopound_force_inches(kilopound_force_inches: float):
        """
        Create a new instance of Torque from a value in kilopound_force_inches.

        

        :param meters: The Torque value in kilopound_force_inches.
        :type kilopound_force_inches: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilopound_force_inches, TorqueUnits.KilopoundForceInch)

    
    @staticmethod
    def from_megapound_force_inches(megapound_force_inches: float):
        """
        Create a new instance of Torque from a value in megapound_force_inches.

        

        :param meters: The Torque value in megapound_force_inches.
        :type megapound_force_inches: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(megapound_force_inches, TorqueUnits.MegapoundForceInch)

    
    @staticmethod
    def from_kilopound_force_feet(kilopound_force_feet: float):
        """
        Create a new instance of Torque from a value in kilopound_force_feet.

        

        :param meters: The Torque value in kilopound_force_feet.
        :type kilopound_force_feet: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(kilopound_force_feet, TorqueUnits.KilopoundForceFoot)

    
    @staticmethod
    def from_megapound_force_feet(megapound_force_feet: float):
        """
        Create a new instance of Torque from a value in megapound_force_feet.

        

        :param meters: The Torque value in megapound_force_feet.
        :type megapound_force_feet: float
        :return: A new instance of Torque.
        :rtype: Torque
        """
        return Torque(megapound_force_feet, TorqueUnits.MegapoundForceFoot)

    
    @property
    def newton_millimeters(self) -> float:
        """
        
        """
        if self.__newton_millimeters != None:
            return self.__newton_millimeters
        self.__newton_millimeters = self.__convert_from_base(TorqueUnits.NewtonMillimeter)
        return self.__newton_millimeters

    
    @property
    def newton_centimeters(self) -> float:
        """
        
        """
        if self.__newton_centimeters != None:
            return self.__newton_centimeters
        self.__newton_centimeters = self.__convert_from_base(TorqueUnits.NewtonCentimeter)
        return self.__newton_centimeters

    
    @property
    def newton_meters(self) -> float:
        """
        
        """
        if self.__newton_meters != None:
            return self.__newton_meters
        self.__newton_meters = self.__convert_from_base(TorqueUnits.NewtonMeter)
        return self.__newton_meters

    
    @property
    def poundal_feet(self) -> float:
        """
        
        """
        if self.__poundal_feet != None:
            return self.__poundal_feet
        self.__poundal_feet = self.__convert_from_base(TorqueUnits.PoundalFoot)
        return self.__poundal_feet

    
    @property
    def pound_force_inches(self) -> float:
        """
        
        """
        if self.__pound_force_inches != None:
            return self.__pound_force_inches
        self.__pound_force_inches = self.__convert_from_base(TorqueUnits.PoundForceInch)
        return self.__pound_force_inches

    
    @property
    def pound_force_feet(self) -> float:
        """
        
        """
        if self.__pound_force_feet != None:
            return self.__pound_force_feet
        self.__pound_force_feet = self.__convert_from_base(TorqueUnits.PoundForceFoot)
        return self.__pound_force_feet

    
    @property
    def gram_force_millimeters(self) -> float:
        """
        
        """
        if self.__gram_force_millimeters != None:
            return self.__gram_force_millimeters
        self.__gram_force_millimeters = self.__convert_from_base(TorqueUnits.GramForceMillimeter)
        return self.__gram_force_millimeters

    
    @property
    def gram_force_centimeters(self) -> float:
        """
        
        """
        if self.__gram_force_centimeters != None:
            return self.__gram_force_centimeters
        self.__gram_force_centimeters = self.__convert_from_base(TorqueUnits.GramForceCentimeter)
        return self.__gram_force_centimeters

    
    @property
    def gram_force_meters(self) -> float:
        """
        
        """
        if self.__gram_force_meters != None:
            return self.__gram_force_meters
        self.__gram_force_meters = self.__convert_from_base(TorqueUnits.GramForceMeter)
        return self.__gram_force_meters

    
    @property
    def kilogram_force_millimeters(self) -> float:
        """
        
        """
        if self.__kilogram_force_millimeters != None:
            return self.__kilogram_force_millimeters
        self.__kilogram_force_millimeters = self.__convert_from_base(TorqueUnits.KilogramForceMillimeter)
        return self.__kilogram_force_millimeters

    
    @property
    def kilogram_force_centimeters(self) -> float:
        """
        
        """
        if self.__kilogram_force_centimeters != None:
            return self.__kilogram_force_centimeters
        self.__kilogram_force_centimeters = self.__convert_from_base(TorqueUnits.KilogramForceCentimeter)
        return self.__kilogram_force_centimeters

    
    @property
    def kilogram_force_meters(self) -> float:
        """
        
        """
        if self.__kilogram_force_meters != None:
            return self.__kilogram_force_meters
        self.__kilogram_force_meters = self.__convert_from_base(TorqueUnits.KilogramForceMeter)
        return self.__kilogram_force_meters

    
    @property
    def tonne_force_millimeters(self) -> float:
        """
        
        """
        if self.__tonne_force_millimeters != None:
            return self.__tonne_force_millimeters
        self.__tonne_force_millimeters = self.__convert_from_base(TorqueUnits.TonneForceMillimeter)
        return self.__tonne_force_millimeters

    
    @property
    def tonne_force_centimeters(self) -> float:
        """
        
        """
        if self.__tonne_force_centimeters != None:
            return self.__tonne_force_centimeters
        self.__tonne_force_centimeters = self.__convert_from_base(TorqueUnits.TonneForceCentimeter)
        return self.__tonne_force_centimeters

    
    @property
    def tonne_force_meters(self) -> float:
        """
        
        """
        if self.__tonne_force_meters != None:
            return self.__tonne_force_meters
        self.__tonne_force_meters = self.__convert_from_base(TorqueUnits.TonneForceMeter)
        return self.__tonne_force_meters

    
    @property
    def kilonewton_millimeters(self) -> float:
        """
        
        """
        if self.__kilonewton_millimeters != None:
            return self.__kilonewton_millimeters
        self.__kilonewton_millimeters = self.__convert_from_base(TorqueUnits.KilonewtonMillimeter)
        return self.__kilonewton_millimeters

    
    @property
    def meganewton_millimeters(self) -> float:
        """
        
        """
        if self.__meganewton_millimeters != None:
            return self.__meganewton_millimeters
        self.__meganewton_millimeters = self.__convert_from_base(TorqueUnits.MeganewtonMillimeter)
        return self.__meganewton_millimeters

    
    @property
    def kilonewton_centimeters(self) -> float:
        """
        
        """
        if self.__kilonewton_centimeters != None:
            return self.__kilonewton_centimeters
        self.__kilonewton_centimeters = self.__convert_from_base(TorqueUnits.KilonewtonCentimeter)
        return self.__kilonewton_centimeters

    
    @property
    def meganewton_centimeters(self) -> float:
        """
        
        """
        if self.__meganewton_centimeters != None:
            return self.__meganewton_centimeters
        self.__meganewton_centimeters = self.__convert_from_base(TorqueUnits.MeganewtonCentimeter)
        return self.__meganewton_centimeters

    
    @property
    def kilonewton_meters(self) -> float:
        """
        
        """
        if self.__kilonewton_meters != None:
            return self.__kilonewton_meters
        self.__kilonewton_meters = self.__convert_from_base(TorqueUnits.KilonewtonMeter)
        return self.__kilonewton_meters

    
    @property
    def meganewton_meters(self) -> float:
        """
        
        """
        if self.__meganewton_meters != None:
            return self.__meganewton_meters
        self.__meganewton_meters = self.__convert_from_base(TorqueUnits.MeganewtonMeter)
        return self.__meganewton_meters

    
    @property
    def kilopound_force_inches(self) -> float:
        """
        
        """
        if self.__kilopound_force_inches != None:
            return self.__kilopound_force_inches
        self.__kilopound_force_inches = self.__convert_from_base(TorqueUnits.KilopoundForceInch)
        return self.__kilopound_force_inches

    
    @property
    def megapound_force_inches(self) -> float:
        """
        
        """
        if self.__megapound_force_inches != None:
            return self.__megapound_force_inches
        self.__megapound_force_inches = self.__convert_from_base(TorqueUnits.MegapoundForceInch)
        return self.__megapound_force_inches

    
    @property
    def kilopound_force_feet(self) -> float:
        """
        
        """
        if self.__kilopound_force_feet != None:
            return self.__kilopound_force_feet
        self.__kilopound_force_feet = self.__convert_from_base(TorqueUnits.KilopoundForceFoot)
        return self.__kilopound_force_feet

    
    @property
    def megapound_force_feet(self) -> float:
        """
        
        """
        if self.__megapound_force_feet != None:
            return self.__megapound_force_feet
        self.__megapound_force_feet = self.__convert_from_base(TorqueUnits.MegapoundForceFoot)
        return self.__megapound_force_feet

    
    def to_string(self, unit: TorqueUnits = TorqueUnits.NewtonMeter) -> string:
        """
        Format the Torque to string.
        Note! the default format for Torque is NewtonMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == TorqueUnits.NewtonMillimeter:
            return f"""{self.newton_millimeters} N·mm"""
        
        if unit == TorqueUnits.NewtonCentimeter:
            return f"""{self.newton_centimeters} N·cm"""
        
        if unit == TorqueUnits.NewtonMeter:
            return f"""{self.newton_meters} N·m"""
        
        if unit == TorqueUnits.PoundalFoot:
            return f"""{self.poundal_feet} pdl·ft"""
        
        if unit == TorqueUnits.PoundForceInch:
            return f"""{self.pound_force_inches} lbf·in"""
        
        if unit == TorqueUnits.PoundForceFoot:
            return f"""{self.pound_force_feet} lbf·ft"""
        
        if unit == TorqueUnits.GramForceMillimeter:
            return f"""{self.gram_force_millimeters} gf·mm"""
        
        if unit == TorqueUnits.GramForceCentimeter:
            return f"""{self.gram_force_centimeters} gf·cm"""
        
        if unit == TorqueUnits.GramForceMeter:
            return f"""{self.gram_force_meters} gf·m"""
        
        if unit == TorqueUnits.KilogramForceMillimeter:
            return f"""{self.kilogram_force_millimeters} kgf·mm"""
        
        if unit == TorqueUnits.KilogramForceCentimeter:
            return f"""{self.kilogram_force_centimeters} kgf·cm"""
        
        if unit == TorqueUnits.KilogramForceMeter:
            return f"""{self.kilogram_force_meters} kgf·m"""
        
        if unit == TorqueUnits.TonneForceMillimeter:
            return f"""{self.tonne_force_millimeters} tf·mm"""
        
        if unit == TorqueUnits.TonneForceCentimeter:
            return f"""{self.tonne_force_centimeters} tf·cm"""
        
        if unit == TorqueUnits.TonneForceMeter:
            return f"""{self.tonne_force_meters} tf·m"""
        
        if unit == TorqueUnits.KilonewtonMillimeter:
            return f"""{self.kilonewton_millimeters} """
        
        if unit == TorqueUnits.MeganewtonMillimeter:
            return f"""{self.meganewton_millimeters} """
        
        if unit == TorqueUnits.KilonewtonCentimeter:
            return f"""{self.kilonewton_centimeters} """
        
        if unit == TorqueUnits.MeganewtonCentimeter:
            return f"""{self.meganewton_centimeters} """
        
        if unit == TorqueUnits.KilonewtonMeter:
            return f"""{self.kilonewton_meters} """
        
        if unit == TorqueUnits.MeganewtonMeter:
            return f"""{self.meganewton_meters} """
        
        if unit == TorqueUnits.KilopoundForceInch:
            return f"""{self.kilopound_force_inches} """
        
        if unit == TorqueUnits.MegapoundForceInch:
            return f"""{self.megapound_force_inches} """
        
        if unit == TorqueUnits.KilopoundForceFoot:
            return f"""{self.kilopound_force_feet} """
        
        if unit == TorqueUnits.MegapoundForceFoot:
            return f"""{self.megapound_force_feet} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: TorqueUnits = TorqueUnits.NewtonMeter) -> string:
        """
        Get Torque unit abbreviation.
        Note! the default abbreviation for Torque is NewtonMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TorqueUnits.NewtonMillimeter:
            return """N·mm"""
        
        if unit_abbreviation == TorqueUnits.NewtonCentimeter:
            return """N·cm"""
        
        if unit_abbreviation == TorqueUnits.NewtonMeter:
            return """N·m"""
        
        if unit_abbreviation == TorqueUnits.PoundalFoot:
            return """pdl·ft"""
        
        if unit_abbreviation == TorqueUnits.PoundForceInch:
            return """lbf·in"""
        
        if unit_abbreviation == TorqueUnits.PoundForceFoot:
            return """lbf·ft"""
        
        if unit_abbreviation == TorqueUnits.GramForceMillimeter:
            return """gf·mm"""
        
        if unit_abbreviation == TorqueUnits.GramForceCentimeter:
            return """gf·cm"""
        
        if unit_abbreviation == TorqueUnits.GramForceMeter:
            return """gf·m"""
        
        if unit_abbreviation == TorqueUnits.KilogramForceMillimeter:
            return """kgf·mm"""
        
        if unit_abbreviation == TorqueUnits.KilogramForceCentimeter:
            return """kgf·cm"""
        
        if unit_abbreviation == TorqueUnits.KilogramForceMeter:
            return """kgf·m"""
        
        if unit_abbreviation == TorqueUnits.TonneForceMillimeter:
            return """tf·mm"""
        
        if unit_abbreviation == TorqueUnits.TonneForceCentimeter:
            return """tf·cm"""
        
        if unit_abbreviation == TorqueUnits.TonneForceMeter:
            return """tf·m"""
        
        if unit_abbreviation == TorqueUnits.KilonewtonMillimeter:
            return """"""
        
        if unit_abbreviation == TorqueUnits.MeganewtonMillimeter:
            return """"""
        
        if unit_abbreviation == TorqueUnits.KilonewtonCentimeter:
            return """"""
        
        if unit_abbreviation == TorqueUnits.MeganewtonCentimeter:
            return """"""
        
        if unit_abbreviation == TorqueUnits.KilonewtonMeter:
            return """"""
        
        if unit_abbreviation == TorqueUnits.MeganewtonMeter:
            return """"""
        
        if unit_abbreviation == TorqueUnits.KilopoundForceInch:
            return """"""
        
        if unit_abbreviation == TorqueUnits.MegapoundForceInch:
            return """"""
        
        if unit_abbreviation == TorqueUnits.KilopoundForceFoot:
            return """"""
        
        if unit_abbreviation == TorqueUnits.MegapoundForceFoot:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for +: 'Torque' and '{}'".format(type(other).__name__))
        return Torque(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for *: 'Torque' and '{}'".format(type(other).__name__))
        return Torque(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for -: 'Torque' and '{}'".format(type(other).__name__))
        return Torque(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for /: 'Torque' and '{}'".format(type(other).__name__))
        return Torque(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for %: 'Torque' and '{}'".format(type(other).__name__))
        return Torque(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for **: 'Torque' and '{}'".format(type(other).__name__))
        return Torque(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for ==: 'Torque' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for <: 'Torque' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for >: 'Torque' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for <=: 'Torque' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Torque):
            raise TypeError("unsupported operand type(s) for >=: 'Torque' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class MagneticFieldUnits(Enum):
        """
            MagneticFieldUnits enumeration
        """
        
        Tesla = 'tesla'
        """
            
        """
        
        Gauss = 'gauss'
        """
            
        """
        
        Nanotesla = 'nanotesla'
        """
            
        """
        
        Microtesla = 'microtesla'
        """
            
        """
        
        Millitesla = 'millitesla'
        """
            
        """
        
        Milligauss = 'milligauss'
        """
            
        """
        

class MagneticField:
    """
    A magnetic field is a force field that is created by moving electric charges (electric currents) and magnetic dipoles, and exerts a force on other nearby moving charges and magnetic dipoles.

    Args:
        value (float): The value.
        from_unit (MagneticFieldUnits): The MagneticField unit to create from, The default unit is Tesla
    """
    def __init__(self, value: float, from_unit: MagneticFieldUnits = MagneticFieldUnits.Tesla):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__teslas = None
        
        self.__gausses = None
        
        self.__nanoteslas = None
        
        self.__microteslas = None
        
        self.__milliteslas = None
        
        self.__milligausses = None
        

    def __convert_from_base(self, from_unit: MagneticFieldUnits) -> float:
        value = self.__value
        
        if from_unit == MagneticFieldUnits.Tesla:
            return (value)
        
        if from_unit == MagneticFieldUnits.Gauss:
            return (value * 1e4)
        
        if from_unit == MagneticFieldUnits.Nanotesla:
            return ((value) / 1e-09)
        
        if from_unit == MagneticFieldUnits.Microtesla:
            return ((value) / 1e-06)
        
        if from_unit == MagneticFieldUnits.Millitesla:
            return ((value) / 0.001)
        
        if from_unit == MagneticFieldUnits.Milligauss:
            return ((value * 1e4) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MagneticFieldUnits) -> float:
        
        if to_unit == MagneticFieldUnits.Tesla:
            return (value)
        
        if to_unit == MagneticFieldUnits.Gauss:
            return (value / 1e4)
        
        if to_unit == MagneticFieldUnits.Nanotesla:
            return ((value) * 1e-09)
        
        if to_unit == MagneticFieldUnits.Microtesla:
            return ((value) * 1e-06)
        
        if to_unit == MagneticFieldUnits.Millitesla:
            return ((value) * 0.001)
        
        if to_unit == MagneticFieldUnits.Milligauss:
            return ((value / 1e4) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_teslas(teslas: float):
        """
        Create a new instance of MagneticField from a value in teslas.

        

        :param meters: The MagneticField value in teslas.
        :type teslas: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(teslas, MagneticFieldUnits.Tesla)

    
    @staticmethod
    def from_gausses(gausses: float):
        """
        Create a new instance of MagneticField from a value in gausses.

        

        :param meters: The MagneticField value in gausses.
        :type gausses: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(gausses, MagneticFieldUnits.Gauss)

    
    @staticmethod
    def from_nanoteslas(nanoteslas: float):
        """
        Create a new instance of MagneticField from a value in nanoteslas.

        

        :param meters: The MagneticField value in nanoteslas.
        :type nanoteslas: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(nanoteslas, MagneticFieldUnits.Nanotesla)

    
    @staticmethod
    def from_microteslas(microteslas: float):
        """
        Create a new instance of MagneticField from a value in microteslas.

        

        :param meters: The MagneticField value in microteslas.
        :type microteslas: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(microteslas, MagneticFieldUnits.Microtesla)

    
    @staticmethod
    def from_milliteslas(milliteslas: float):
        """
        Create a new instance of MagneticField from a value in milliteslas.

        

        :param meters: The MagneticField value in milliteslas.
        :type milliteslas: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(milliteslas, MagneticFieldUnits.Millitesla)

    
    @staticmethod
    def from_milligausses(milligausses: float):
        """
        Create a new instance of MagneticField from a value in milligausses.

        

        :param meters: The MagneticField value in milligausses.
        :type milligausses: float
        :return: A new instance of MagneticField.
        :rtype: MagneticField
        """
        return MagneticField(milligausses, MagneticFieldUnits.Milligauss)

    
    @property
    def teslas(self) -> float:
        """
        
        """
        if self.__teslas != None:
            return self.__teslas
        self.__teslas = self.__convert_from_base(MagneticFieldUnits.Tesla)
        return self.__teslas

    
    @property
    def gausses(self) -> float:
        """
        
        """
        if self.__gausses != None:
            return self.__gausses
        self.__gausses = self.__convert_from_base(MagneticFieldUnits.Gauss)
        return self.__gausses

    
    @property
    def nanoteslas(self) -> float:
        """
        
        """
        if self.__nanoteslas != None:
            return self.__nanoteslas
        self.__nanoteslas = self.__convert_from_base(MagneticFieldUnits.Nanotesla)
        return self.__nanoteslas

    
    @property
    def microteslas(self) -> float:
        """
        
        """
        if self.__microteslas != None:
            return self.__microteslas
        self.__microteslas = self.__convert_from_base(MagneticFieldUnits.Microtesla)
        return self.__microteslas

    
    @property
    def milliteslas(self) -> float:
        """
        
        """
        if self.__milliteslas != None:
            return self.__milliteslas
        self.__milliteslas = self.__convert_from_base(MagneticFieldUnits.Millitesla)
        return self.__milliteslas

    
    @property
    def milligausses(self) -> float:
        """
        
        """
        if self.__milligausses != None:
            return self.__milligausses
        self.__milligausses = self.__convert_from_base(MagneticFieldUnits.Milligauss)
        return self.__milligausses

    
    def to_string(self, unit: MagneticFieldUnits = MagneticFieldUnits.Tesla) -> string:
        """
        Format the MagneticField to string.
        Note! the default format for MagneticField is Tesla.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == MagneticFieldUnits.Tesla:
            return f"""{self.teslas} T"""
        
        if unit == MagneticFieldUnits.Gauss:
            return f"""{self.gausses} G"""
        
        if unit == MagneticFieldUnits.Nanotesla:
            return f"""{self.nanoteslas} """
        
        if unit == MagneticFieldUnits.Microtesla:
            return f"""{self.microteslas} """
        
        if unit == MagneticFieldUnits.Millitesla:
            return f"""{self.milliteslas} """
        
        if unit == MagneticFieldUnits.Milligauss:
            return f"""{self.milligausses} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: MagneticFieldUnits = MagneticFieldUnits.Tesla) -> string:
        """
        Get MagneticField unit abbreviation.
        Note! the default abbreviation for MagneticField is Tesla.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MagneticFieldUnits.Tesla:
            return """T"""
        
        if unit_abbreviation == MagneticFieldUnits.Gauss:
            return """G"""
        
        if unit_abbreviation == MagneticFieldUnits.Nanotesla:
            return """"""
        
        if unit_abbreviation == MagneticFieldUnits.Microtesla:
            return """"""
        
        if unit_abbreviation == MagneticFieldUnits.Millitesla:
            return """"""
        
        if unit_abbreviation == MagneticFieldUnits.Milligauss:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for +: 'MagneticField' and '{}'".format(type(other).__name__))
        return MagneticField(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for *: 'MagneticField' and '{}'".format(type(other).__name__))
        return MagneticField(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for -: 'MagneticField' and '{}'".format(type(other).__name__))
        return MagneticField(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for /: 'MagneticField' and '{}'".format(type(other).__name__))
        return MagneticField(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for %: 'MagneticField' and '{}'".format(type(other).__name__))
        return MagneticField(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for **: 'MagneticField' and '{}'".format(type(other).__name__))
        return MagneticField(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for ==: 'MagneticField' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for <: 'MagneticField' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for >: 'MagneticField' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for <=: 'MagneticField' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, MagneticField):
            raise TypeError("unsupported operand type(s) for >=: 'MagneticField' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class BrakeSpecificFuelConsumptionUnits(Enum):
        """
            BrakeSpecificFuelConsumptionUnits enumeration
        """
        
        GramPerKiloWattHour = 'gram_per_kilo_watt_hour'
        """
            
        """
        
        KilogramPerJoule = 'kilogram_per_joule'
        """
            
        """
        
        PoundPerMechanicalHorsepowerHour = 'pound_per_mechanical_horsepower_hour'
        """
            The pound per horse power hour uses mechanical horse power and the imperial pound
        """
        

class BrakeSpecificFuelConsumption:
    """
    Brake specific fuel consumption (BSFC) is a measure of the fuel efficiency of any prime mover that burns fuel and produces rotational, or shaft, power. It is typically used for comparing the efficiency of internal combustion engines with a shaft output.

    Args:
        value (float): The value.
        from_unit (BrakeSpecificFuelConsumptionUnits): The BrakeSpecificFuelConsumption unit to create from, The default unit is KilogramPerJoule
    """
    def __init__(self, value: float, from_unit: BrakeSpecificFuelConsumptionUnits = BrakeSpecificFuelConsumptionUnits.KilogramPerJoule):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__grams_per_kilo_watt_hour = None
        
        self.__kilograms_per_joule = None
        
        self.__pounds_per_mechanical_horsepower_hour = None
        

    def __convert_from_base(self, from_unit: BrakeSpecificFuelConsumptionUnits) -> float:
        value = self.__value
        
        if from_unit == BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour:
            return (value * 3.6e9)
        
        if from_unit == BrakeSpecificFuelConsumptionUnits.KilogramPerJoule:
            return (value)
        
        if from_unit == BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour:
            return (value / 1.689659410672e-7)
        
        return None


    def __convert_to_base(self, value: float, to_unit: BrakeSpecificFuelConsumptionUnits) -> float:
        
        if to_unit == BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour:
            return (value / 3.6e9)
        
        if to_unit == BrakeSpecificFuelConsumptionUnits.KilogramPerJoule:
            return (value)
        
        if to_unit == BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour:
            return (value * 1.689659410672e-7)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_grams_per_kilo_watt_hour(grams_per_kilo_watt_hour: float):
        """
        Create a new instance of BrakeSpecificFuelConsumption from a value in grams_per_kilo_watt_hour.

        

        :param meters: The BrakeSpecificFuelConsumption value in grams_per_kilo_watt_hour.
        :type grams_per_kilo_watt_hour: float
        :return: A new instance of BrakeSpecificFuelConsumption.
        :rtype: BrakeSpecificFuelConsumption
        """
        return BrakeSpecificFuelConsumption(grams_per_kilo_watt_hour, BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour)

    
    @staticmethod
    def from_kilograms_per_joule(kilograms_per_joule: float):
        """
        Create a new instance of BrakeSpecificFuelConsumption from a value in kilograms_per_joule.

        

        :param meters: The BrakeSpecificFuelConsumption value in kilograms_per_joule.
        :type kilograms_per_joule: float
        :return: A new instance of BrakeSpecificFuelConsumption.
        :rtype: BrakeSpecificFuelConsumption
        """
        return BrakeSpecificFuelConsumption(kilograms_per_joule, BrakeSpecificFuelConsumptionUnits.KilogramPerJoule)

    
    @staticmethod
    def from_pounds_per_mechanical_horsepower_hour(pounds_per_mechanical_horsepower_hour: float):
        """
        Create a new instance of BrakeSpecificFuelConsumption from a value in pounds_per_mechanical_horsepower_hour.

        The pound per horse power hour uses mechanical horse power and the imperial pound

        :param meters: The BrakeSpecificFuelConsumption value in pounds_per_mechanical_horsepower_hour.
        :type pounds_per_mechanical_horsepower_hour: float
        :return: A new instance of BrakeSpecificFuelConsumption.
        :rtype: BrakeSpecificFuelConsumption
        """
        return BrakeSpecificFuelConsumption(pounds_per_mechanical_horsepower_hour, BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour)

    
    @property
    def grams_per_kilo_watt_hour(self) -> float:
        """
        
        """
        if self.__grams_per_kilo_watt_hour != None:
            return self.__grams_per_kilo_watt_hour
        self.__grams_per_kilo_watt_hour = self.__convert_from_base(BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour)
        return self.__grams_per_kilo_watt_hour

    
    @property
    def kilograms_per_joule(self) -> float:
        """
        
        """
        if self.__kilograms_per_joule != None:
            return self.__kilograms_per_joule
        self.__kilograms_per_joule = self.__convert_from_base(BrakeSpecificFuelConsumptionUnits.KilogramPerJoule)
        return self.__kilograms_per_joule

    
    @property
    def pounds_per_mechanical_horsepower_hour(self) -> float:
        """
        The pound per horse power hour uses mechanical horse power and the imperial pound
        """
        if self.__pounds_per_mechanical_horsepower_hour != None:
            return self.__pounds_per_mechanical_horsepower_hour
        self.__pounds_per_mechanical_horsepower_hour = self.__convert_from_base(BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour)
        return self.__pounds_per_mechanical_horsepower_hour

    
    def to_string(self, unit: BrakeSpecificFuelConsumptionUnits = BrakeSpecificFuelConsumptionUnits.KilogramPerJoule) -> string:
        """
        Format the BrakeSpecificFuelConsumption to string.
        Note! the default format for BrakeSpecificFuelConsumption is KilogramPerJoule.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour:
            return f"""{self.grams_per_kilo_watt_hour} g/kWh"""
        
        if unit == BrakeSpecificFuelConsumptionUnits.KilogramPerJoule:
            return f"""{self.kilograms_per_joule} kg/J"""
        
        if unit == BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour:
            return f"""{self.pounds_per_mechanical_horsepower_hour} lb/hph"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: BrakeSpecificFuelConsumptionUnits = BrakeSpecificFuelConsumptionUnits.KilogramPerJoule) -> string:
        """
        Get BrakeSpecificFuelConsumption unit abbreviation.
        Note! the default abbreviation for BrakeSpecificFuelConsumption is KilogramPerJoule.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == BrakeSpecificFuelConsumptionUnits.GramPerKiloWattHour:
            return """g/kWh"""
        
        if unit_abbreviation == BrakeSpecificFuelConsumptionUnits.KilogramPerJoule:
            return """kg/J"""
        
        if unit_abbreviation == BrakeSpecificFuelConsumptionUnits.PoundPerMechanicalHorsepowerHour:
            return """lb/hph"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for +: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return BrakeSpecificFuelConsumption(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for *: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return BrakeSpecificFuelConsumption(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for -: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return BrakeSpecificFuelConsumption(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for /: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return BrakeSpecificFuelConsumption(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for %: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return BrakeSpecificFuelConsumption(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for **: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return BrakeSpecificFuelConsumption(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for ==: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for <: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for >: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for <=: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, BrakeSpecificFuelConsumption):
            raise TypeError("unsupported operand type(s) for >=: 'BrakeSpecificFuelConsumption' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class BitRateUnits(Enum):
        """
            BitRateUnits enumeration
        """
        
        BitPerSecond = 'bit_per_second'
        """
            
        """
        
        BytePerSecond = 'byte_per_second'
        """
            
        """
        
        KilobitPerSecond = 'kilobit_per_second'
        """
            
        """
        
        MegabitPerSecond = 'megabit_per_second'
        """
            
        """
        
        GigabitPerSecond = 'gigabit_per_second'
        """
            
        """
        
        TerabitPerSecond = 'terabit_per_second'
        """
            
        """
        
        PetabitPerSecond = 'petabit_per_second'
        """
            
        """
        
        ExabitPerSecond = 'exabit_per_second'
        """
            
        """
        
        KilobytePerSecond = 'kilobyte_per_second'
        """
            
        """
        
        MegabytePerSecond = 'megabyte_per_second'
        """
            
        """
        
        GigabytePerSecond = 'gigabyte_per_second'
        """
            
        """
        
        TerabytePerSecond = 'terabyte_per_second'
        """
            
        """
        
        PetabytePerSecond = 'petabyte_per_second'
        """
            
        """
        
        ExabytePerSecond = 'exabyte_per_second'
        """
            
        """
        

class BitRate:
    """
    In telecommunications and computing, bit rate is the number of bits that are conveyed or processed per unit of time.

    Args:
        value (float): The value.
        from_unit (BitRateUnits): The BitRate unit to create from, The default unit is BitPerSecond
    """
    def __init__(self, value: float, from_unit: BitRateUnits = BitRateUnits.BitPerSecond):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__bits_per_second = None
        
        self.__bytes_per_second = None
        
        self.__kilobits_per_second = None
        
        self.__megabits_per_second = None
        
        self.__gigabits_per_second = None
        
        self.__terabits_per_second = None
        
        self.__petabits_per_second = None
        
        self.__exabits_per_second = None
        
        self.__kilobytes_per_second = None
        
        self.__megabytes_per_second = None
        
        self.__gigabytes_per_second = None
        
        self.__terabytes_per_second = None
        
        self.__petabytes_per_second = None
        
        self.__exabytes_per_second = None
        

    def __convert_from_base(self, from_unit: BitRateUnits) -> float:
        value = self.__value
        
        if from_unit == BitRateUnits.BitPerSecond:
            return (value)
        
        if from_unit == BitRateUnits.BytePerSecond:
            return (value / 8)
        
        if from_unit == BitRateUnits.KilobitPerSecond:
            return ((value) / 1000.0)
        
        if from_unit == BitRateUnits.MegabitPerSecond:
            return ((value) / 1000000.0)
        
        if from_unit == BitRateUnits.GigabitPerSecond:
            return ((value) / 1000000000.0)
        
        if from_unit == BitRateUnits.TerabitPerSecond:
            return ((value) / 1000000000000.0)
        
        if from_unit == BitRateUnits.PetabitPerSecond:
            return ((value) / 1000000000000000.0)
        
        if from_unit == BitRateUnits.ExabitPerSecond:
            return ((value) / 1e+18)
        
        if from_unit == BitRateUnits.KilobytePerSecond:
            return ((value / 8) / 1000.0)
        
        if from_unit == BitRateUnits.MegabytePerSecond:
            return ((value / 8) / 1000000.0)
        
        if from_unit == BitRateUnits.GigabytePerSecond:
            return ((value / 8) / 1000000000.0)
        
        if from_unit == BitRateUnits.TerabytePerSecond:
            return ((value / 8) / 1000000000000.0)
        
        if from_unit == BitRateUnits.PetabytePerSecond:
            return ((value / 8) / 1000000000000000.0)
        
        if from_unit == BitRateUnits.ExabytePerSecond:
            return ((value / 8) / 1e+18)
        
        return None


    def __convert_to_base(self, value: float, to_unit: BitRateUnits) -> float:
        
        if to_unit == BitRateUnits.BitPerSecond:
            return (value)
        
        if to_unit == BitRateUnits.BytePerSecond:
            return (value * 8)
        
        if to_unit == BitRateUnits.KilobitPerSecond:
            return ((value) * 1000.0)
        
        if to_unit == BitRateUnits.MegabitPerSecond:
            return ((value) * 1000000.0)
        
        if to_unit == BitRateUnits.GigabitPerSecond:
            return ((value) * 1000000000.0)
        
        if to_unit == BitRateUnits.TerabitPerSecond:
            return ((value) * 1000000000000.0)
        
        if to_unit == BitRateUnits.PetabitPerSecond:
            return ((value) * 1000000000000000.0)
        
        if to_unit == BitRateUnits.ExabitPerSecond:
            return ((value) * 1e+18)
        
        if to_unit == BitRateUnits.KilobytePerSecond:
            return ((value * 8) * 1000.0)
        
        if to_unit == BitRateUnits.MegabytePerSecond:
            return ((value * 8) * 1000000.0)
        
        if to_unit == BitRateUnits.GigabytePerSecond:
            return ((value * 8) * 1000000000.0)
        
        if to_unit == BitRateUnits.TerabytePerSecond:
            return ((value * 8) * 1000000000000.0)
        
        if to_unit == BitRateUnits.PetabytePerSecond:
            return ((value * 8) * 1000000000000000.0)
        
        if to_unit == BitRateUnits.ExabytePerSecond:
            return ((value * 8) * 1e+18)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_bits_per_second(bits_per_second: float):
        """
        Create a new instance of BitRate from a value in bits_per_second.

        

        :param meters: The BitRate value in bits_per_second.
        :type bits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(bits_per_second, BitRateUnits.BitPerSecond)

    
    @staticmethod
    def from_bytes_per_second(bytes_per_second: float):
        """
        Create a new instance of BitRate from a value in bytes_per_second.

        

        :param meters: The BitRate value in bytes_per_second.
        :type bytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(bytes_per_second, BitRateUnits.BytePerSecond)

    
    @staticmethod
    def from_kilobits_per_second(kilobits_per_second: float):
        """
        Create a new instance of BitRate from a value in kilobits_per_second.

        

        :param meters: The BitRate value in kilobits_per_second.
        :type kilobits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(kilobits_per_second, BitRateUnits.KilobitPerSecond)

    
    @staticmethod
    def from_megabits_per_second(megabits_per_second: float):
        """
        Create a new instance of BitRate from a value in megabits_per_second.

        

        :param meters: The BitRate value in megabits_per_second.
        :type megabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(megabits_per_second, BitRateUnits.MegabitPerSecond)

    
    @staticmethod
    def from_gigabits_per_second(gigabits_per_second: float):
        """
        Create a new instance of BitRate from a value in gigabits_per_second.

        

        :param meters: The BitRate value in gigabits_per_second.
        :type gigabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(gigabits_per_second, BitRateUnits.GigabitPerSecond)

    
    @staticmethod
    def from_terabits_per_second(terabits_per_second: float):
        """
        Create a new instance of BitRate from a value in terabits_per_second.

        

        :param meters: The BitRate value in terabits_per_second.
        :type terabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(terabits_per_second, BitRateUnits.TerabitPerSecond)

    
    @staticmethod
    def from_petabits_per_second(petabits_per_second: float):
        """
        Create a new instance of BitRate from a value in petabits_per_second.

        

        :param meters: The BitRate value in petabits_per_second.
        :type petabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(petabits_per_second, BitRateUnits.PetabitPerSecond)

    
    @staticmethod
    def from_exabits_per_second(exabits_per_second: float):
        """
        Create a new instance of BitRate from a value in exabits_per_second.

        

        :param meters: The BitRate value in exabits_per_second.
        :type exabits_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(exabits_per_second, BitRateUnits.ExabitPerSecond)

    
    @staticmethod
    def from_kilobytes_per_second(kilobytes_per_second: float):
        """
        Create a new instance of BitRate from a value in kilobytes_per_second.

        

        :param meters: The BitRate value in kilobytes_per_second.
        :type kilobytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(kilobytes_per_second, BitRateUnits.KilobytePerSecond)

    
    @staticmethod
    def from_megabytes_per_second(megabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in megabytes_per_second.

        

        :param meters: The BitRate value in megabytes_per_second.
        :type megabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(megabytes_per_second, BitRateUnits.MegabytePerSecond)

    
    @staticmethod
    def from_gigabytes_per_second(gigabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in gigabytes_per_second.

        

        :param meters: The BitRate value in gigabytes_per_second.
        :type gigabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(gigabytes_per_second, BitRateUnits.GigabytePerSecond)

    
    @staticmethod
    def from_terabytes_per_second(terabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in terabytes_per_second.

        

        :param meters: The BitRate value in terabytes_per_second.
        :type terabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(terabytes_per_second, BitRateUnits.TerabytePerSecond)

    
    @staticmethod
    def from_petabytes_per_second(petabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in petabytes_per_second.

        

        :param meters: The BitRate value in petabytes_per_second.
        :type petabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(petabytes_per_second, BitRateUnits.PetabytePerSecond)

    
    @staticmethod
    def from_exabytes_per_second(exabytes_per_second: float):
        """
        Create a new instance of BitRate from a value in exabytes_per_second.

        

        :param meters: The BitRate value in exabytes_per_second.
        :type exabytes_per_second: float
        :return: A new instance of BitRate.
        :rtype: BitRate
        """
        return BitRate(exabytes_per_second, BitRateUnits.ExabytePerSecond)

    
    @property
    def bits_per_second(self) -> float:
        """
        
        """
        if self.__bits_per_second != None:
            return self.__bits_per_second
        self.__bits_per_second = self.__convert_from_base(BitRateUnits.BitPerSecond)
        return self.__bits_per_second

    
    @property
    def bytes_per_second(self) -> float:
        """
        
        """
        if self.__bytes_per_second != None:
            return self.__bytes_per_second
        self.__bytes_per_second = self.__convert_from_base(BitRateUnits.BytePerSecond)
        return self.__bytes_per_second

    
    @property
    def kilobits_per_second(self) -> float:
        """
        
        """
        if self.__kilobits_per_second != None:
            return self.__kilobits_per_second
        self.__kilobits_per_second = self.__convert_from_base(BitRateUnits.KilobitPerSecond)
        return self.__kilobits_per_second

    
    @property
    def megabits_per_second(self) -> float:
        """
        
        """
        if self.__megabits_per_second != None:
            return self.__megabits_per_second
        self.__megabits_per_second = self.__convert_from_base(BitRateUnits.MegabitPerSecond)
        return self.__megabits_per_second

    
    @property
    def gigabits_per_second(self) -> float:
        """
        
        """
        if self.__gigabits_per_second != None:
            return self.__gigabits_per_second
        self.__gigabits_per_second = self.__convert_from_base(BitRateUnits.GigabitPerSecond)
        return self.__gigabits_per_second

    
    @property
    def terabits_per_second(self) -> float:
        """
        
        """
        if self.__terabits_per_second != None:
            return self.__terabits_per_second
        self.__terabits_per_second = self.__convert_from_base(BitRateUnits.TerabitPerSecond)
        return self.__terabits_per_second

    
    @property
    def petabits_per_second(self) -> float:
        """
        
        """
        if self.__petabits_per_second != None:
            return self.__petabits_per_second
        self.__petabits_per_second = self.__convert_from_base(BitRateUnits.PetabitPerSecond)
        return self.__petabits_per_second

    
    @property
    def exabits_per_second(self) -> float:
        """
        
        """
        if self.__exabits_per_second != None:
            return self.__exabits_per_second
        self.__exabits_per_second = self.__convert_from_base(BitRateUnits.ExabitPerSecond)
        return self.__exabits_per_second

    
    @property
    def kilobytes_per_second(self) -> float:
        """
        
        """
        if self.__kilobytes_per_second != None:
            return self.__kilobytes_per_second
        self.__kilobytes_per_second = self.__convert_from_base(BitRateUnits.KilobytePerSecond)
        return self.__kilobytes_per_second

    
    @property
    def megabytes_per_second(self) -> float:
        """
        
        """
        if self.__megabytes_per_second != None:
            return self.__megabytes_per_second
        self.__megabytes_per_second = self.__convert_from_base(BitRateUnits.MegabytePerSecond)
        return self.__megabytes_per_second

    
    @property
    def gigabytes_per_second(self) -> float:
        """
        
        """
        if self.__gigabytes_per_second != None:
            return self.__gigabytes_per_second
        self.__gigabytes_per_second = self.__convert_from_base(BitRateUnits.GigabytePerSecond)
        return self.__gigabytes_per_second

    
    @property
    def terabytes_per_second(self) -> float:
        """
        
        """
        if self.__terabytes_per_second != None:
            return self.__terabytes_per_second
        self.__terabytes_per_second = self.__convert_from_base(BitRateUnits.TerabytePerSecond)
        return self.__terabytes_per_second

    
    @property
    def petabytes_per_second(self) -> float:
        """
        
        """
        if self.__petabytes_per_second != None:
            return self.__petabytes_per_second
        self.__petabytes_per_second = self.__convert_from_base(BitRateUnits.PetabytePerSecond)
        return self.__petabytes_per_second

    
    @property
    def exabytes_per_second(self) -> float:
        """
        
        """
        if self.__exabytes_per_second != None:
            return self.__exabytes_per_second
        self.__exabytes_per_second = self.__convert_from_base(BitRateUnits.ExabytePerSecond)
        return self.__exabytes_per_second

    
    def to_string(self, unit: BitRateUnits = BitRateUnits.BitPerSecond) -> string:
        """
        Format the BitRate to string.
        Note! the default format for BitRate is BitPerSecond.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == BitRateUnits.BitPerSecond:
            return f"""{self.bits_per_second} bit/s"""
        
        if unit == BitRateUnits.BytePerSecond:
            return f"""{self.bytes_per_second} B/s"""
        
        if unit == BitRateUnits.KilobitPerSecond:
            return f"""{self.kilobits_per_second} """
        
        if unit == BitRateUnits.MegabitPerSecond:
            return f"""{self.megabits_per_second} """
        
        if unit == BitRateUnits.GigabitPerSecond:
            return f"""{self.gigabits_per_second} """
        
        if unit == BitRateUnits.TerabitPerSecond:
            return f"""{self.terabits_per_second} """
        
        if unit == BitRateUnits.PetabitPerSecond:
            return f"""{self.petabits_per_second} """
        
        if unit == BitRateUnits.ExabitPerSecond:
            return f"""{self.exabits_per_second} """
        
        if unit == BitRateUnits.KilobytePerSecond:
            return f"""{self.kilobytes_per_second} """
        
        if unit == BitRateUnits.MegabytePerSecond:
            return f"""{self.megabytes_per_second} """
        
        if unit == BitRateUnits.GigabytePerSecond:
            return f"""{self.gigabytes_per_second} """
        
        if unit == BitRateUnits.TerabytePerSecond:
            return f"""{self.terabytes_per_second} """
        
        if unit == BitRateUnits.PetabytePerSecond:
            return f"""{self.petabytes_per_second} """
        
        if unit == BitRateUnits.ExabytePerSecond:
            return f"""{self.exabytes_per_second} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: BitRateUnits = BitRateUnits.BitPerSecond) -> string:
        """
        Get BitRate unit abbreviation.
        Note! the default abbreviation for BitRate is BitPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == BitRateUnits.BitPerSecond:
            return """bit/s"""
        
        if unit_abbreviation == BitRateUnits.BytePerSecond:
            return """B/s"""
        
        if unit_abbreviation == BitRateUnits.KilobitPerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.MegabitPerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.GigabitPerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.TerabitPerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.PetabitPerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.ExabitPerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.KilobytePerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.MegabytePerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.GigabytePerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.TerabytePerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.PetabytePerSecond:
            return """"""
        
        if unit_abbreviation == BitRateUnits.ExabytePerSecond:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for +: 'BitRate' and '{}'".format(type(other).__name__))
        return BitRate(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for *: 'BitRate' and '{}'".format(type(other).__name__))
        return BitRate(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for -: 'BitRate' and '{}'".format(type(other).__name__))
        return BitRate(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for /: 'BitRate' and '{}'".format(type(other).__name__))
        return BitRate(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for %: 'BitRate' and '{}'".format(type(other).__name__))
        return BitRate(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for **: 'BitRate' and '{}'".format(type(other).__name__))
        return BitRate(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for ==: 'BitRate' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for <: 'BitRate' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for >: 'BitRate' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for <=: 'BitRate' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, BitRate):
            raise TypeError("unsupported operand type(s) for >=: 'BitRate' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ElectricPotentialAcUnits(Enum):
        """
            ElectricPotentialAcUnits enumeration
        """
        
        VoltAc = 'volt_ac'
        """
            
        """
        
        MicrovoltAc = 'microvolt_ac'
        """
            
        """
        
        MillivoltAc = 'millivolt_ac'
        """
            
        """
        
        KilovoltAc = 'kilovolt_ac'
        """
            
        """
        
        MegavoltAc = 'megavolt_ac'
        """
            
        """
        

class ElectricPotentialAc:
    """
    The Electric Potential of a system known to use Alternating Current.

    Args:
        value (float): The value.
        from_unit (ElectricPotentialAcUnits): The ElectricPotentialAc unit to create from, The default unit is VoltAc
    """
    def __init__(self, value: float, from_unit: ElectricPotentialAcUnits = ElectricPotentialAcUnits.VoltAc):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__volts_ac = None
        
        self.__microvolts_ac = None
        
        self.__millivolts_ac = None
        
        self.__kilovolts_ac = None
        
        self.__megavolts_ac = None
        

    def __convert_from_base(self, from_unit: ElectricPotentialAcUnits) -> float:
        value = self.__value
        
        if from_unit == ElectricPotentialAcUnits.VoltAc:
            return (value)
        
        if from_unit == ElectricPotentialAcUnits.MicrovoltAc:
            return ((value) / 1e-06)
        
        if from_unit == ElectricPotentialAcUnits.MillivoltAc:
            return ((value) / 0.001)
        
        if from_unit == ElectricPotentialAcUnits.KilovoltAc:
            return ((value) / 1000.0)
        
        if from_unit == ElectricPotentialAcUnits.MegavoltAc:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricPotentialAcUnits) -> float:
        
        if to_unit == ElectricPotentialAcUnits.VoltAc:
            return (value)
        
        if to_unit == ElectricPotentialAcUnits.MicrovoltAc:
            return ((value) * 1e-06)
        
        if to_unit == ElectricPotentialAcUnits.MillivoltAc:
            return ((value) * 0.001)
        
        if to_unit == ElectricPotentialAcUnits.KilovoltAc:
            return ((value) * 1000.0)
        
        if to_unit == ElectricPotentialAcUnits.MegavoltAc:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_volts_ac(volts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in volts_ac.

        

        :param meters: The ElectricPotentialAc value in volts_ac.
        :type volts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(volts_ac, ElectricPotentialAcUnits.VoltAc)

    
    @staticmethod
    def from_microvolts_ac(microvolts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in microvolts_ac.

        

        :param meters: The ElectricPotentialAc value in microvolts_ac.
        :type microvolts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(microvolts_ac, ElectricPotentialAcUnits.MicrovoltAc)

    
    @staticmethod
    def from_millivolts_ac(millivolts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in millivolts_ac.

        

        :param meters: The ElectricPotentialAc value in millivolts_ac.
        :type millivolts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(millivolts_ac, ElectricPotentialAcUnits.MillivoltAc)

    
    @staticmethod
    def from_kilovolts_ac(kilovolts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in kilovolts_ac.

        

        :param meters: The ElectricPotentialAc value in kilovolts_ac.
        :type kilovolts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(kilovolts_ac, ElectricPotentialAcUnits.KilovoltAc)

    
    @staticmethod
    def from_megavolts_ac(megavolts_ac: float):
        """
        Create a new instance of ElectricPotentialAc from a value in megavolts_ac.

        

        :param meters: The ElectricPotentialAc value in megavolts_ac.
        :type megavolts_ac: float
        :return: A new instance of ElectricPotentialAc.
        :rtype: ElectricPotentialAc
        """
        return ElectricPotentialAc(megavolts_ac, ElectricPotentialAcUnits.MegavoltAc)

    
    @property
    def volts_ac(self) -> float:
        """
        
        """
        if self.__volts_ac != None:
            return self.__volts_ac
        self.__volts_ac = self.__convert_from_base(ElectricPotentialAcUnits.VoltAc)
        return self.__volts_ac

    
    @property
    def microvolts_ac(self) -> float:
        """
        
        """
        if self.__microvolts_ac != None:
            return self.__microvolts_ac
        self.__microvolts_ac = self.__convert_from_base(ElectricPotentialAcUnits.MicrovoltAc)
        return self.__microvolts_ac

    
    @property
    def millivolts_ac(self) -> float:
        """
        
        """
        if self.__millivolts_ac != None:
            return self.__millivolts_ac
        self.__millivolts_ac = self.__convert_from_base(ElectricPotentialAcUnits.MillivoltAc)
        return self.__millivolts_ac

    
    @property
    def kilovolts_ac(self) -> float:
        """
        
        """
        if self.__kilovolts_ac != None:
            return self.__kilovolts_ac
        self.__kilovolts_ac = self.__convert_from_base(ElectricPotentialAcUnits.KilovoltAc)
        return self.__kilovolts_ac

    
    @property
    def megavolts_ac(self) -> float:
        """
        
        """
        if self.__megavolts_ac != None:
            return self.__megavolts_ac
        self.__megavolts_ac = self.__convert_from_base(ElectricPotentialAcUnits.MegavoltAc)
        return self.__megavolts_ac

    
    def to_string(self, unit: ElectricPotentialAcUnits = ElectricPotentialAcUnits.VoltAc) -> string:
        """
        Format the ElectricPotentialAc to string.
        Note! the default format for ElectricPotentialAc is VoltAc.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ElectricPotentialAcUnits.VoltAc:
            return f"""{self.volts_ac} Vac"""
        
        if unit == ElectricPotentialAcUnits.MicrovoltAc:
            return f"""{self.microvolts_ac} """
        
        if unit == ElectricPotentialAcUnits.MillivoltAc:
            return f"""{self.millivolts_ac} """
        
        if unit == ElectricPotentialAcUnits.KilovoltAc:
            return f"""{self.kilovolts_ac} """
        
        if unit == ElectricPotentialAcUnits.MegavoltAc:
            return f"""{self.megavolts_ac} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricPotentialAcUnits = ElectricPotentialAcUnits.VoltAc) -> string:
        """
        Get ElectricPotentialAc unit abbreviation.
        Note! the default abbreviation for ElectricPotentialAc is VoltAc.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricPotentialAcUnits.VoltAc:
            return """Vac"""
        
        if unit_abbreviation == ElectricPotentialAcUnits.MicrovoltAc:
            return """"""
        
        if unit_abbreviation == ElectricPotentialAcUnits.MillivoltAc:
            return """"""
        
        if unit_abbreviation == ElectricPotentialAcUnits.KilovoltAc:
            return """"""
        
        if unit_abbreviation == ElectricPotentialAcUnits.MegavoltAc:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for +: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return ElectricPotentialAc(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for *: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return ElectricPotentialAc(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for -: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return ElectricPotentialAc(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for /: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return ElectricPotentialAc(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for %: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return ElectricPotentialAc(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for **: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return ElectricPotentialAc(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for ==: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for <: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for >: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for <=: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ElectricPotentialAc):
            raise TypeError("unsupported operand type(s) for >=: 'ElectricPotentialAc' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class IrradiationUnits(Enum):
        """
            IrradiationUnits enumeration
        """
        
        JoulePerSquareMeter = 'joule_per_square_meter'
        """
            
        """
        
        JoulePerSquareCentimeter = 'joule_per_square_centimeter'
        """
            
        """
        
        JoulePerSquareMillimeter = 'joule_per_square_millimeter'
        """
            
        """
        
        WattHourPerSquareMeter = 'watt_hour_per_square_meter'
        """
            
        """
        
        KilojoulePerSquareMeter = 'kilojoule_per_square_meter'
        """
            
        """
        
        MillijoulePerSquareCentimeter = 'millijoule_per_square_centimeter'
        """
            
        """
        
        KilowattHourPerSquareMeter = 'kilowatt_hour_per_square_meter'
        """
            
        """
        

class Irradiation:
    """
    Irradiation is the process by which an object is exposed to radiation. The exposure can originate from various sources, including natural sources.

    Args:
        value (float): The value.
        from_unit (IrradiationUnits): The Irradiation unit to create from, The default unit is JoulePerSquareMeter
    """
    def __init__(self, value: float, from_unit: IrradiationUnits = IrradiationUnits.JoulePerSquareMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_square_meter = None
        
        self.__joules_per_square_centimeter = None
        
        self.__joules_per_square_millimeter = None
        
        self.__watt_hours_per_square_meter = None
        
        self.__kilojoules_per_square_meter = None
        
        self.__millijoules_per_square_centimeter = None
        
        self.__kilowatt_hours_per_square_meter = None
        

    def __convert_from_base(self, from_unit: IrradiationUnits) -> float:
        value = self.__value
        
        if from_unit == IrradiationUnits.JoulePerSquareMeter:
            return (value)
        
        if from_unit == IrradiationUnits.JoulePerSquareCentimeter:
            return (value / 1e4)
        
        if from_unit == IrradiationUnits.JoulePerSquareMillimeter:
            return (value / 1e6)
        
        if from_unit == IrradiationUnits.WattHourPerSquareMeter:
            return (value / 3600)
        
        if from_unit == IrradiationUnits.KilojoulePerSquareMeter:
            return ((value) / 1000.0)
        
        if from_unit == IrradiationUnits.MillijoulePerSquareCentimeter:
            return ((value / 1e4) / 0.001)
        
        if from_unit == IrradiationUnits.KilowattHourPerSquareMeter:
            return ((value / 3600) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: IrradiationUnits) -> float:
        
        if to_unit == IrradiationUnits.JoulePerSquareMeter:
            return (value)
        
        if to_unit == IrradiationUnits.JoulePerSquareCentimeter:
            return (value * 1e4)
        
        if to_unit == IrradiationUnits.JoulePerSquareMillimeter:
            return (value * 1e6)
        
        if to_unit == IrradiationUnits.WattHourPerSquareMeter:
            return (value * 3600)
        
        if to_unit == IrradiationUnits.KilojoulePerSquareMeter:
            return ((value) * 1000.0)
        
        if to_unit == IrradiationUnits.MillijoulePerSquareCentimeter:
            return ((value * 1e4) * 0.001)
        
        if to_unit == IrradiationUnits.KilowattHourPerSquareMeter:
            return ((value * 3600) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_joules_per_square_meter(joules_per_square_meter: float):
        """
        Create a new instance of Irradiation from a value in joules_per_square_meter.

        

        :param meters: The Irradiation value in joules_per_square_meter.
        :type joules_per_square_meter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(joules_per_square_meter, IrradiationUnits.JoulePerSquareMeter)

    
    @staticmethod
    def from_joules_per_square_centimeter(joules_per_square_centimeter: float):
        """
        Create a new instance of Irradiation from a value in joules_per_square_centimeter.

        

        :param meters: The Irradiation value in joules_per_square_centimeter.
        :type joules_per_square_centimeter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(joules_per_square_centimeter, IrradiationUnits.JoulePerSquareCentimeter)

    
    @staticmethod
    def from_joules_per_square_millimeter(joules_per_square_millimeter: float):
        """
        Create a new instance of Irradiation from a value in joules_per_square_millimeter.

        

        :param meters: The Irradiation value in joules_per_square_millimeter.
        :type joules_per_square_millimeter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(joules_per_square_millimeter, IrradiationUnits.JoulePerSquareMillimeter)

    
    @staticmethod
    def from_watt_hours_per_square_meter(watt_hours_per_square_meter: float):
        """
        Create a new instance of Irradiation from a value in watt_hours_per_square_meter.

        

        :param meters: The Irradiation value in watt_hours_per_square_meter.
        :type watt_hours_per_square_meter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(watt_hours_per_square_meter, IrradiationUnits.WattHourPerSquareMeter)

    
    @staticmethod
    def from_kilojoules_per_square_meter(kilojoules_per_square_meter: float):
        """
        Create a new instance of Irradiation from a value in kilojoules_per_square_meter.

        

        :param meters: The Irradiation value in kilojoules_per_square_meter.
        :type kilojoules_per_square_meter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(kilojoules_per_square_meter, IrradiationUnits.KilojoulePerSquareMeter)

    
    @staticmethod
    def from_millijoules_per_square_centimeter(millijoules_per_square_centimeter: float):
        """
        Create a new instance of Irradiation from a value in millijoules_per_square_centimeter.

        

        :param meters: The Irradiation value in millijoules_per_square_centimeter.
        :type millijoules_per_square_centimeter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(millijoules_per_square_centimeter, IrradiationUnits.MillijoulePerSquareCentimeter)

    
    @staticmethod
    def from_kilowatt_hours_per_square_meter(kilowatt_hours_per_square_meter: float):
        """
        Create a new instance of Irradiation from a value in kilowatt_hours_per_square_meter.

        

        :param meters: The Irradiation value in kilowatt_hours_per_square_meter.
        :type kilowatt_hours_per_square_meter: float
        :return: A new instance of Irradiation.
        :rtype: Irradiation
        """
        return Irradiation(kilowatt_hours_per_square_meter, IrradiationUnits.KilowattHourPerSquareMeter)

    
    @property
    def joules_per_square_meter(self) -> float:
        """
        
        """
        if self.__joules_per_square_meter != None:
            return self.__joules_per_square_meter
        self.__joules_per_square_meter = self.__convert_from_base(IrradiationUnits.JoulePerSquareMeter)
        return self.__joules_per_square_meter

    
    @property
    def joules_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__joules_per_square_centimeter != None:
            return self.__joules_per_square_centimeter
        self.__joules_per_square_centimeter = self.__convert_from_base(IrradiationUnits.JoulePerSquareCentimeter)
        return self.__joules_per_square_centimeter

    
    @property
    def joules_per_square_millimeter(self) -> float:
        """
        
        """
        if self.__joules_per_square_millimeter != None:
            return self.__joules_per_square_millimeter
        self.__joules_per_square_millimeter = self.__convert_from_base(IrradiationUnits.JoulePerSquareMillimeter)
        return self.__joules_per_square_millimeter

    
    @property
    def watt_hours_per_square_meter(self) -> float:
        """
        
        """
        if self.__watt_hours_per_square_meter != None:
            return self.__watt_hours_per_square_meter
        self.__watt_hours_per_square_meter = self.__convert_from_base(IrradiationUnits.WattHourPerSquareMeter)
        return self.__watt_hours_per_square_meter

    
    @property
    def kilojoules_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilojoules_per_square_meter != None:
            return self.__kilojoules_per_square_meter
        self.__kilojoules_per_square_meter = self.__convert_from_base(IrradiationUnits.KilojoulePerSquareMeter)
        return self.__kilojoules_per_square_meter

    
    @property
    def millijoules_per_square_centimeter(self) -> float:
        """
        
        """
        if self.__millijoules_per_square_centimeter != None:
            return self.__millijoules_per_square_centimeter
        self.__millijoules_per_square_centimeter = self.__convert_from_base(IrradiationUnits.MillijoulePerSquareCentimeter)
        return self.__millijoules_per_square_centimeter

    
    @property
    def kilowatt_hours_per_square_meter(self) -> float:
        """
        
        """
        if self.__kilowatt_hours_per_square_meter != None:
            return self.__kilowatt_hours_per_square_meter
        self.__kilowatt_hours_per_square_meter = self.__convert_from_base(IrradiationUnits.KilowattHourPerSquareMeter)
        return self.__kilowatt_hours_per_square_meter

    
    def to_string(self, unit: IrradiationUnits = IrradiationUnits.JoulePerSquareMeter) -> string:
        """
        Format the Irradiation to string.
        Note! the default format for Irradiation is JoulePerSquareMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == IrradiationUnits.JoulePerSquareMeter:
            return f"""{self.joules_per_square_meter} J/m²"""
        
        if unit == IrradiationUnits.JoulePerSquareCentimeter:
            return f"""{self.joules_per_square_centimeter} J/cm²"""
        
        if unit == IrradiationUnits.JoulePerSquareMillimeter:
            return f"""{self.joules_per_square_millimeter} J/mm²"""
        
        if unit == IrradiationUnits.WattHourPerSquareMeter:
            return f"""{self.watt_hours_per_square_meter} Wh/m²"""
        
        if unit == IrradiationUnits.KilojoulePerSquareMeter:
            return f"""{self.kilojoules_per_square_meter} """
        
        if unit == IrradiationUnits.MillijoulePerSquareCentimeter:
            return f"""{self.millijoules_per_square_centimeter} """
        
        if unit == IrradiationUnits.KilowattHourPerSquareMeter:
            return f"""{self.kilowatt_hours_per_square_meter} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: IrradiationUnits = IrradiationUnits.JoulePerSquareMeter) -> string:
        """
        Get Irradiation unit abbreviation.
        Note! the default abbreviation for Irradiation is JoulePerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == IrradiationUnits.JoulePerSquareMeter:
            return """J/m²"""
        
        if unit_abbreviation == IrradiationUnits.JoulePerSquareCentimeter:
            return """J/cm²"""
        
        if unit_abbreviation == IrradiationUnits.JoulePerSquareMillimeter:
            return """J/mm²"""
        
        if unit_abbreviation == IrradiationUnits.WattHourPerSquareMeter:
            return """Wh/m²"""
        
        if unit_abbreviation == IrradiationUnits.KilojoulePerSquareMeter:
            return """"""
        
        if unit_abbreviation == IrradiationUnits.MillijoulePerSquareCentimeter:
            return """"""
        
        if unit_abbreviation == IrradiationUnits.KilowattHourPerSquareMeter:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for +: 'Irradiation' and '{}'".format(type(other).__name__))
        return Irradiation(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for *: 'Irradiation' and '{}'".format(type(other).__name__))
        return Irradiation(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for -: 'Irradiation' and '{}'".format(type(other).__name__))
        return Irradiation(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for /: 'Irradiation' and '{}'".format(type(other).__name__))
        return Irradiation(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for %: 'Irradiation' and '{}'".format(type(other).__name__))
        return Irradiation(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for **: 'Irradiation' and '{}'".format(type(other).__name__))
        return Irradiation(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for ==: 'Irradiation' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for <: 'Irradiation' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for >: 'Irradiation' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for <=: 'Irradiation' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Irradiation):
            raise TypeError("unsupported operand type(s) for >=: 'Irradiation' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value